@@ -1,29 +1,17 @@
 /*
- *   Copyright 2009 by Alan Alpert <alan.alpert@nokia.com>
- *   Copyright 2010 by Ménard Alexis <menard@kde.org>
- *   Copyright 2010 by Marco MArtin <mart@kde.org>
- *   Copyright 2013 by Sebastian Kügler <sebas@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2009 Alan Alpert <alan.alpert@nokia.com>
+    SPDX-FileCopyrightText: 2010 Ménard Alexis <menard@kde.org>
+    SPDX-FileCopyrightText: 2010 Marco MArtin <mart@kde.org>
+    SPDX-FileCopyrightText: 2013 Sebastian Kügler <sebas@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef DATASOURCE_H
 #define DATASOURCE_H
 
 #include <QObject>
+#include <QScopedPointer>
 #include <QtQml>
 #include <QQmlPropertyMap>
 #include <QQmlParserStatus>
@@ -58,8 +46,8 @@ public:
 
     explicit DataSource(QObject *parent = nullptr);
 
-    void classBegin() Q_DECL_OVERRIDE;
-    void componentComplete() Q_DECL_OVERRIDE;
+    void classBegin() override;
+    void componentComplete() override;
 
     /**
      * true if the connection to the Plasma DataEngine is valid
@@ -188,7 +176,7 @@ private:
     QQmlPropertyMap *m_data = nullptr;
     QQmlPropertyMap *m_models = nullptr;
     Plasma::DataEngine *m_dataEngine = nullptr;
-    Plasma::DataEngineConsumer *m_dataEngineConsumer = nullptr;
+    QScopedPointer<Plasma::DataEngineConsumer> m_dataEngineConsumer;
     QStringList m_sources;
     QStringList m_connectedSources;
     QStringList m_oldSources;
