@@ -1,23 +1,10 @@
-/***************************************************************************
- *   Copyright 2013 Marco Martin <mart@kde.org>                            *
- *   Copyright 2014 Sebastian Kügler <sebas@kde.org>                       *
- *   Copyright 2014 David Edmundson <davidedmunsdon@kde.org>               *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2013 Marco Martin <mart@kde.org>
+    SPDX-FileCopyrightText: 2014 Sebastian Kügler <sebas@kde.org>
+    SPDX-FileCopyrightText: 2014 David Edmundson <davidedmunsdon@kde.org>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "units.h"
 
@@ -30,10 +17,9 @@
 #include <QFontMetrics>
 #include <cmath>
 
-#include <KConfigWatcher>
 #include <KIconLoader>
 
-const int defaultLongDuration = 120;
+const int defaultLongDuration = 250;
 
 
 SharedAppFilter::SharedAppFilter(QObject *parent)
@@ -272,7 +258,17 @@ int Units::longDuration() const
 
 int Units::shortDuration() const
 {
-    return qMax(1, m_longDuration / 5);
+    return qMax(1, qRound(m_longDuration * 0.6));
+}
+
+int Units::veryShortDuration() const
+{
+	return qRound(m_longDuration * 0.3);
+}
+
+int Units::veryLongDuration() const
+{
+    return m_longDuration * 2;
 }
 
 #include "moc_units.cpp"
