@@ -1,24 +1,11 @@
 /*
- *   Copyright 2005 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
- *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2005 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "applet.h"
 #include "private/applet_p.h"
@@ -32,18 +19,18 @@
 #include <QList>
 #include <QAbstractButton>
 #include <QMessageBox>
+#include <QMetaEnum>
 
-#include <kactioncollection.h>
-#include <kauthorized.h>
-#include <kcolorscheme.h>
-#include <kdesktopfile.h>
+#include <KActionCollection>
+#include <KAuthorized>
+#include <KColorScheme>
+#include <KDesktopFile>
 #include <QDebug>
-#include <kglobalaccel.h>
-#include <kplugininfo.h>
-#include <klocalizedstring.h>
-#include <kservice.h>
+#include <KGlobalAccel>
+#include <KLocalizedString>
+#include <KService>
 #include <KConfigLoader>
-#include <kwindowsystem.h>
+#include <KWindowSystem>
 
 #include "containment.h"
 #include "corona.h"
@@ -141,6 +128,10 @@ Applet::~Applet()
     //let people know that i will die
     emit appletDeleted(this);
 
+    // ConfigLoader is deleted when AppletPrivate closes not Applet
+    // It saves on closure and emits a signal.
+    // disconnect early to avoid a crash. See  411221
+    disconnect(d->configLoader, SIGNAL(configChanged()), this, SLOT(propagateConfigChanged()));
     delete d;
 }
 
@@ -219,6 +210,21 @@ void Applet::restore(KConfigGroup &group)
         //TODO: implement; the shortcut
     }
     */
+
+    // User background hints
+    // TODO support flags in the config
+    QByteArray hintsString = config().readEntry("UserBackgroundHints", QString()).toUtf8();
+    QMetaEnum hintEnum = QMetaEnum::fromType<Plasma::Types::BackgroundHints>();
+    bool ok;
+    int value = hintEnum.keyToValue(hintsString.constData(), &ok);
+    if (ok) {
+        d->userBackgroundHints = Plasma::Types::BackgroundHints(value);
+        d->userBackgroundHintsInitialized = true;
+        emit userBackgroundHintsChanged();
+        if (d->backgroundHints & Plasma::Types::ConfigurableBackground) {
+            emit effectiveBackgroundHintsChanged();
+        }
+    }
 }
 
 void Applet::setLaunchErrorMessage(const QString &message)
@@ -395,6 +401,64 @@ void Applet::setBusy(bool busy)
     emit busyChanged(busy);
 }
 
+Plasma::Types::BackgroundHints Applet::backgroundHints() const
+{
+    return d->backgroundHints;
+}
+
+void Applet::setBackgroundHints(Plasma::Types::BackgroundHints hint)
+{
+    if (d->backgroundHints == hint) {
+        return;
+    }
+
+    Plasma::Types::BackgroundHints oldeffectiveHints = effectiveBackgroundHints();
+
+    d->backgroundHints = hint;
+    emit backgroundHintsChanged();
+
+    if (oldeffectiveHints != effectiveBackgroundHints()) {
+        emit effectiveBackgroundHintsChanged();
+    }
+}
+
+Plasma::Types::BackgroundHints Applet::effectiveBackgroundHints() const
+{
+    if (d->userBackgroundHintsInitialized
+        && (d->backgroundHints & Plasma::Types::ConfigurableBackground)) {
+        return d->userBackgroundHints;
+    } else {
+        return d->backgroundHints;
+    }
+}
+
+Plasma::Types::BackgroundHints Applet::userBackgroundHints() const
+{
+    return d->userBackgroundHints;
+}
+
+void Applet::setUserBackgroundHints(Plasma::Types::BackgroundHints hint)
+{
+    if (d->userBackgroundHints == hint && d->userBackgroundHintsInitialized) {
+        return;
+    }
+
+    d->userBackgroundHints = hint;
+    d->userBackgroundHintsInitialized = true;
+    QMetaEnum hintEnum = QMetaEnum::fromType<Plasma::Types::BackgroundHints>();
+    config().writeEntry("UserBackgroundHints", hintEnum.valueToKey(d->userBackgroundHints));
+    if (containment() && containment()->corona()) {
+        containment()->corona()->requestConfigSync();
+    }
+
+    emit userBackgroundHintsChanged();
+
+    if (d->backgroundHints & Plasma::Types::ConfigurableBackground) {
+        emit effectiveBackgroundHintsChanged();
+    }
+}
+
+
 KPluginInfo Applet::pluginInfo() const
 {
     return KPluginInfo(d->appletDescription);
@@ -640,6 +704,13 @@ Types::FormFactor Applet::formFactor() const
     return c ? c->d->formFactor : Plasma::Types::Planar;
 }
 
+Types::ContainmentDisplayHints Applet::containmentDisplayHints() const
+{
+    Containment *c = containment();
+
+    return c ? c->d->containmentDisplayHints : Plasma::Types::NoContainmentDisplayHint;
+}
+
 Containment *Applet::containment() const
 {
     Containment *c = qobject_cast<Containment *>(const_cast<Applet *>(this));
