@@ -1,24 +1,11 @@
 /*
- *   Copyright 2007 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
- *   Copyright 2009 Chani Armitage <chani@kde.org>
- *   Copyright 2012 Marco Martin <notmart@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
+    SPDX-FileCopyrightText: 2012 Marco Martin <notmart@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "containment.h"
 #include "private/containment_p.h"
@@ -31,21 +18,14 @@
 #include <QContextMenuEvent>
 #include <QMimeData>
 #include <QPainter>
-#include <qtemporaryfile.h>
-#include <qmimedatabase.h>
+#include <QTemporaryFile>
+#include <QMimeDatabase>
 
-#include <QDebug>
-#include <kauthorized.h>
-#include <klocalizedstring.h>
+#include <KAuthorized>
+#include <KLocalizedString>
 #include <KConfigSkeleton>
 #include <KConfigLoader>
 
-#if !PLASMA_NO_KIO
-#include "kio/jobclasses.h" // for KIO::JobFlags
-#include "kio/job.h"
-#include "kio/scheduler.h"
-#endif
-
 #include "containmentactions.h"
 #include "corona.h"
 #include "pluginloader.h"
@@ -124,12 +104,17 @@ void Containment::init()
     //btw, do we really want to use title() when it's a desktopcontainment?
     QAction *closeApplet = actions()->action(QStringLiteral("remove"));
     if (closeApplet) {
-        closeApplet->setText(i18nc("%1 is the name of the applet", "Remove this %1", title()));
+        closeApplet->setText(i18nc("%1 is the name of the applet", "Remove %1", title()));
     }
 
     QAction *configAction = actions()->action(QStringLiteral("configure"));
     if (configAction) {
-        configAction->setText(i18nc("%1 is the name of the applet", "%1 Settings", title()));
+        if (d->type == Types::PanelContainment || d->type == Types::CustomPanelContainment) {
+            configAction->setText(i18nc("%1 is the name of the containment", "Edit %1...", title()));
+            configAction->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
+        } else {
+            configAction->setText(i18nc("%1 is the name of the applet", "Configure %1...", title()));
+        }
     }
 
     QAction *appletBrowserAction = actions()->action(QStringLiteral("add widgets"));
@@ -187,13 +172,14 @@ void Containment::restore(KConfigGroup &group)
     restoreContents(group);
     setImmutability((Types::ImmutabilityType)group.readEntry("immutability", (int)Types::Mutable));
 
-    if (isContainment()) {
+    if (isContainment() && KAuthorized::authorize(QStringLiteral("plasma/containment_actions"))) {
         KConfigGroup cfg = KConfigGroup(corona()->config(), "ActionPlugins");
         cfg = KConfigGroup(&cfg, QString::number(containmentType()));
 
         //qCDebug(LOG_PLASMA) << cfg.keyList();
         if (cfg.exists()) {
-            foreach (const QString &key, cfg.keyList()) {
+            const auto keyList =  cfg.keyList();
+            for (const QString &key : keyList) {
                 //qCDebug(LOG_PLASMA) << "loading" << key;
                 setContainmentActions(key, cfg.readEntry(key, QString()));
             }
@@ -215,7 +201,8 @@ void Containment::restore(KConfigGroup &group)
             }
             if (defaultActionsCfg.isValid()) {
                 defaultActionsCfg = KConfigGroup(&defaultActionsCfg, "ContainmentActions");
-                foreach (const QString &key, defaultActionsCfg.keyList()) {
+                const auto keyList = defaultActionsCfg.keyList();
+                for (const QString &key : keyList) {
                     setContainmentActions(key, defaultActionsCfg.readEntry(key, QString()));
                 }
             }
@@ -260,7 +247,7 @@ void Containment::save(KConfigGroup &g) const
 void Containment::saveContents(KConfigGroup &group) const
 {
     KConfigGroup applets(&group, "Applets");
-    foreach (const Applet *applet, d->applets) {
+    for (const Applet *applet : qAsConst(d->applets)) {
         KConfigGroup appletConfig(&applets, QString::number(applet->id()));
         applet->save(appletConfig);
     }
@@ -272,17 +259,17 @@ void Containment::restoreContents(KConfigGroup &group)
 
     //restore the applets ordered by id
     QStringList groups = applets.groupList();
-    qSort(groups.begin(), groups.end());
+    std::sort(groups.begin(), groups.end());
 
     // Sort the applet configs in order of geometry to ensure that applets
     // are added from left to right or top to bottom for a panel containment
     QList<KConfigGroup> appletConfigs;
-    foreach (const QString &appletGroup, groups) {
+    for (const QString &appletGroup : qAsConst(groups)) {
         //qCDebug(LOG_PLASMA) << "reading from applet group" << appletGroup;
         KConfigGroup appletConfig(&applets, appletGroup);
         appletConfigs.append(appletConfig);
     }
-    qStableSort(appletConfigs.begin(), appletConfigs.end(), appletConfigLessThan);
+    std::stable_sort(appletConfigs.begin(), appletConfigs.end(), appletConfigLessThan);
 
     QMutableListIterator<KConfigGroup> it(appletConfigs);
     while (it.hasNext()) {
@@ -301,7 +288,8 @@ void Containment::restoreContents(KConfigGroup &group)
     if (Containment::applets().isEmpty()) {
         d->appletsUiReady = true;
     }
-    foreach (Applet *applet, Containment::applets()) {
+    const auto lstApplets = Containment::applets();
+    for (Applet *applet : lstApplets) {
         if (!applet->pluginMetaData().isValid()) {
             applet->updateConstraints(Plasma::Types::UiReadyConstraint);
         }
@@ -355,6 +343,16 @@ void Containment::setFormFactor(Types::FormFactor formFactor)
     emit formFactorChanged(formFactor);
 }
 
+void Containment::setContainmentDisplayHints(Types::ContainmentDisplayHints hints)
+{
+    if (d->containmentDisplayHints == hints) {
+        return;
+    }
+
+    d->containmentDisplayHints = hints;
+    emit containmentDisplayHintsChanged(hints);
+}
+
 void Containment::setLocation(Types::Location location)
 {
     if (d->location == location) {
@@ -363,7 +361,7 @@ void Containment::setLocation(Types::Location location)
 
     d->location = location;
 
-    foreach (Applet *applet, d->applets) {
+    for (Applet *applet : qAsConst(d->applets)) {
         applet->updateConstraints(Plasma::Types::LocationConstraint);
     }
 
@@ -408,7 +406,8 @@ void Containment::addApplet(Applet *applet)
     if (currentContainment && currentContainment != this) {
         emit currentContainment->appletRemoved(applet);
 
-        disconnect(applet, 0, currentContainment, 0);
+        disconnect(applet, nullptr, currentContainment, nullptr);
+        connect(currentContainment, nullptr, applet, nullptr);
         KConfigGroup oldConfig = applet->config();
         currentContainment->d->applets.removeAll(applet);
         applet->setParent(this);
@@ -419,7 +418,7 @@ void Containment::addApplet(Applet *applet)
         oldConfig.reparent(&c);
         applet->d->resetConfigurationObject();
 
-        disconnect(applet, SIGNAL(activated()), currentContainment, SIGNAL(activated()));
+        disconnect(applet, &Applet::activated, currentContainment, &Applet::activated);
         //change the group to its configloader, if any
         //FIXME: this is very, very brutal
         if (applet->configScheme()) {
@@ -428,7 +427,8 @@ void Containment::addApplet(Applet *applet)
 
             applet->configScheme()->setCurrentGroup(applet->configScheme()->currentGroup().replace(0, oldGroupPrefix.length(), newGroupPrefix));
 
-            foreach (KConfigSkeletonItem *item, applet->configScheme()->items()) {
+            const auto items = applet->configScheme()->items();
+            for (KConfigSkeletonItem *item : items) {
                 item->setGroup(item->group().replace(0, oldGroupPrefix.length(), newGroupPrefix));
             }
         }
@@ -446,10 +446,11 @@ void Containment::addApplet(Applet *applet)
         d->loadingApplets << applet;
     }
 
-    connect(applet, SIGNAL(configNeedsSaving()), this, SIGNAL(configNeedsSaving()));
+    connect(applet, &Applet::configNeedsSaving, this, &Applet::configNeedsSaving);
     connect(applet, SIGNAL(appletDeleted(Plasma::Applet*)), this, SLOT(appletDeleted(Plasma::Applet*)));
     connect(applet, SIGNAL(statusChanged(Plasma::Types::ItemStatus)), this, SLOT(checkStatus(Plasma::Types::ItemStatus)));
-    connect(applet, SIGNAL(activated()), this, SIGNAL(activated()));
+    connect(applet, &Applet::activated, this, &Applet::activated);
+    connect(this, &Containment::containmentDisplayHintsChanged, applet, &Applet::containmentDisplayHintsChanged);
 
     if (!currentContainment) {
         const bool isNew = applet->d->mainConfigGroup()->entryMap().isEmpty();
@@ -521,13 +522,13 @@ QString Containment::wallpaper() const
 void Containment::setContainmentActions(const QString &trigger, const QString &pluginName)
 {
     KConfigGroup cfg = d->containmentActionsConfig();
-    ContainmentActions *plugin = 0;
+    ContainmentActions *plugin = nullptr;
 
     plugin = containmentActions().value(trigger);
-    if (plugin && plugin->pluginInfo().pluginName() != pluginName) {
+    if (plugin && plugin->metadata().pluginId() != pluginName) {
         containmentActions().remove(trigger);
         delete plugin;
-        plugin = 0;
+        plugin = nullptr;
     }
 
     if (pluginName.isEmpty()) {
