@@ -27,7 +27,7 @@
 namespace Plasma
 {
 
-QHash<ThemePrivate *, QHash<QString, QWeakPointer<FrameData>> > FrameSvgPrivate::s_sharedFrames;
+QHash<ThemePrivate *, QHash<uint, QWeakPointer<FrameData>> > FrameSvgPrivate::s_sharedFrames;
 
 // Any attempt to generate a frame whose width or height is larger than this
 // will be rejected
@@ -61,7 +61,7 @@ void FrameSvg::setImagePath(const QString &path)
     setContainsMultipleImages(true);
     Svg::d->setImagePath(path);
     if (!d->repaintBlocked) {
-        d->updateFrameData();
+        d->updateFrameData(Svg::d->lastModified);
     }
 }
 
@@ -74,7 +74,7 @@ void FrameSvg::setEnabledBorders(const EnabledBorders borders)
     d->enabledBorders = borders;
 
     if (!d->repaintBlocked) {
-        d->updateFrameData();
+        d->updateFrameData(Svg::d->lastModified);
     }
 }
 
@@ -108,7 +108,7 @@ void FrameSvg::setElementPrefix(Plasma::Types::Location location)
 
 void FrameSvg::setElementPrefix(const QString &prefix)
 {
-    if (!hasElement(prefix % QLatin1String("-center"))) {
+    if (prefix.isEmpty() || !hasElement(prefix % QLatin1String("-center"))) {
         d->prefix.clear();
     } else {
         d->prefix = prefix;
@@ -121,7 +121,7 @@ void FrameSvg::setElementPrefix(const QString &prefix)
     d->location = Types::Floating;
 
     if (!d->repaintBlocked) {
-        d->updateFrameData();
+        d->updateFrameData(Svg::d->lastModified);
     }
 }
 
@@ -179,7 +179,7 @@ void FrameSvg::resizeFrame(const QSizeF &size)
     d->pendingFrameSize = size.toSize();
 
     if (!d->repaintBlocked) {
-        d->updateFrameData(FrameSvgPrivate::UpdateFrame);
+        d->updateFrameData(Svg::d->lastModified, FrameSvgPrivate::UpdateFrame);
     }
 }
 
@@ -218,6 +218,32 @@ qreal FrameSvg::marginSize(const Plasma::Types::MarginEdge edge) const
     }
 }
 
+qreal FrameSvg::insetSize(const Plasma::Types::MarginEdge edge) const
+{
+    if (!d->frame) {
+        return .0;
+    }
+
+    if (d->frame->noBorderPadding) {
+        return .0;
+    }
+
+    switch (edge) {
+    case Plasma::Types::TopMargin:
+        return d->frame->insetTopMargin;
+
+    case Plasma::Types::LeftMargin:
+        return d->frame->insetLeftMargin;
+
+    case Plasma::Types::RightMargin:
+        return d->frame->insetRightMargin;
+
+    //Plasma::BottomMargin
+    default:
+        return d->frame->insetBottomMargin;
+    }
+}
+
 qreal FrameSvg::fixedMarginSize(const Plasma::Types::MarginEdge edge) const
 {
     if (!d->frame) {
@@ -270,6 +296,19 @@ void FrameSvg::getFixedMargins(qreal &left, qreal &top, qreal &right, qreal &bot
     bottom = d->frame->fixedBottomMargin;
 }
 
+void FrameSvg::getInset(qreal &left, qreal &top, qreal &right, qreal &bottom) const
+{
+    if (!d->frame || d->frame->noBorderPadding) {
+        left = top = right = bottom = 0;
+        return;
+    }
+
+    top = d->frame->insetTopMargin;
+    left = d->frame->insetLeftMargin;
+    right = d->frame->insetRightMargin;
+    bottom = d->frame->insetBottomMargin;
+}
+
 QRectF FrameSvg::contentsRect() const
 {
     if (d->frame) {
@@ -293,7 +332,7 @@ QRegion FrameSvg::mask() const
         return result;
     }
 
-    QString id = d->cacheId(d->frame.data(), QString());
+    uint id = qHash(d->cacheId(d->frame.data(), QString()), SvgRectsCache::s_seed);
 
     QRegion* obj = d->frame->cachedMasks.object(id);
 
@@ -420,7 +459,7 @@ QPixmap FrameSvgPrivate::alphaMask()
 
 QSharedPointer<FrameData> FrameSvgPrivate::lookupOrCreateMaskFrame(const QSharedPointer<FrameData> &frame, const QString &maskPrefix, const QString &maskRequestedPrefix)
 {
-    const QString key = cacheId(frame.data(), maskPrefix);
+    const uint key = qHash(cacheId(frame.data(), maskPrefix));
     QSharedPointer<FrameData> mask = s_sharedFrames[q->theme()->d].value(key);
 
     // See if we can find a suitable candidate in the shared frames.
@@ -437,6 +476,7 @@ QSharedPointer<FrameData> FrameSvgPrivate::lookupOrCreateMaskFrame(const QShared
     mask->enabledBorders = frame->enabledBorders;
     mask->frameSize = frameSize(frame).toSize();
     mask->cacheId = key;
+    mask->lastModified = frame->lastModified;
     s_sharedFrames[q->theme()->d].insert(key, mask);
 
     return mask;
@@ -448,17 +488,19 @@ void FrameSvgPrivate::generateBackground(const QSharedPointer<FrameData> &frame)
         return;
     }
 
-    const QString id = cacheId(frame.data(), frame->prefix);
+    const uint id = qHash(cacheId(frame.data(), frame->prefix));
 
     bool frameCached = !frame->cachedBackground.isNull();
     bool overlayCached = false;
+    //TODO KF6: Kill Overlays
     const bool overlayAvailable = !frame->prefix.startsWith(QLatin1String("mask-")) && q->hasElement(frame->prefix % QLatin1String("overlay"));
     QPixmap overlay;
     if (q->isUsingRenderingCache()) {
-        frameCached = q->theme()->findInCache(id, frame->cachedBackground) && !frame->cachedBackground.isNull();
+        frameCached = q->theme()->findInCache(QString::number(id), frame->cachedBackground, frame->lastModified) && !frame->cachedBackground.isNull();
 
         if (overlayAvailable) {
-            overlayCached = q->theme()->findInCache(QLatin1String("overlay_") % id, overlay) && !overlay.isNull();
+            const uint overlayId = qHash(cacheId(frame.data(), frame->prefix % QLatin1String("overlay")));
+            overlayCached = q->theme()->findInCache(QString::number(overlayId), overlay, frame->lastModified) && !overlay.isNull();
         }
     }
 
@@ -579,13 +621,13 @@ QRect FrameSvgPrivate::contentGeometry(const QSharedPointer<FrameData> &frame, c
     return contentRect;
 }
 
-void FrameSvgPrivate::updateFrameData(UpdateType updateType)
+void FrameSvgPrivate::updateFrameData(uint lastModified, UpdateType updateType)
 {
     auto fd = frame;
-    QString newKey;
+    uint newKey = 0;
 
     if (fd) {
-        const QString oldKey = fd->cacheId;
+        const uint oldKey = fd->cacheId;
 
         const QString oldPath = fd->imagePath;
         const FrameSvg::EnabledBorders oldBorders = fd->enabledBorders;
@@ -595,7 +637,7 @@ void FrameSvgPrivate::updateFrameData(UpdateType updateType)
         fd->frameSize = pendingFrameSize;
         fd->imagePath = q->imagePath();
 
-        newKey = cacheId(fd.data(), prefix);
+        newKey = qHash(cacheId(fd.data(), prefix));
 
         //reset frame to old values
         fd->enabledBorders = oldBorders;
@@ -629,9 +671,10 @@ void FrameSvgPrivate::updateFrameData(UpdateType updateType)
     fd->enabledBorders = enabledBorders;
     fd->frameSize = pendingFrameSize;
     fd->imagePath = q->imagePath();
+    fd->lastModified = lastModified;
     //was fd just created empty now?
-    if (newKey.isEmpty()) {
-        newKey = cacheId(fd.data(), prefix);
+    if (newKey == 0) {
+        newKey = qHash(cacheId(fd.data(), prefix));
     }
 
     // we know it isn't in s_sharedFrames due to the check above, so insert it now
@@ -711,11 +754,10 @@ void FrameSvgPrivate::paintCorner(QPainter& p, const QSharedPointer<FrameData> &
     }
 }
 
-QString FrameSvgPrivate::cacheId(FrameData *frame, const QString &prefixToSave) const
+SvgPrivate::CacheId FrameSvgPrivate::cacheId(FrameData *frame, const QString &prefixToSave) const
 {
     const QSize size = frameSize(frame).toSize();
-    const QLatin1Char s('_');
-    return QString::number(frame->enabledBorders) % s % QString::number(size.width()) % s % QString::number(size.height()) % s % QString::number(q->scaleFactor()) % s % QString::number(q->devicePixelRatio()) % s % prefixToSave % s % frame->imagePath;
+    return SvgPrivate::CacheId{double(size.width()), double(size.height()), frame->imagePath,  prefixToSave, q->status(), q->devicePixelRatio(), q->scaleFactor(), q->colorGroup(), (uint)frame->enabledBorders, q->Svg::d->lastModified};
 }
 
 void FrameSvgPrivate::cacheFrame(const QString &prefixToSave, const QPixmap &background, const QPixmap &overlay)
@@ -729,15 +771,16 @@ void FrameSvgPrivate::cacheFrame(const QString &prefixToSave, const QPixmap &bac
         return;
     }
 
-    const QString id = cacheId(frame.data(), prefixToSave);
+    const uint id = qHash(cacheId(frame.data(), prefixToSave));
 
     //qCDebug(LOG_PLASMA)<<"Saving to cache frame"<<id;
 
-    q->theme()->insertIntoCache(id, background, QString::number((qint64)q, 16) % prefixToSave);
+    q->theme()->insertIntoCache(QString::number(id), background, QString::number((qint64)q, 16) % prefixToSave);
 
     if (!overlay.isNull()) {
         //insert overlay
-        q->theme()->insertIntoCache(QLatin1String("overlay_") % id, overlay, QString::number((qint64)q, 16) % prefixToSave % QLatin1String("overlay"));
+        const uint overlayId = qHash(cacheId(frame.data(), frame->prefix % QLatin1String("overlay")));
+        q->theme()->insertIntoCache(QString::number(overlayId), overlay, QString::number((qint64)q, 16) % prefixToSave % QLatin1String("overlay"));
     }
 }
 
@@ -769,6 +812,12 @@ void FrameSvgPrivate::updateSizes(FrameData *frame) const
         frame->topMargin = frame->topHeight = 0;
     }
 
+    if (q->hasElement(frame->prefix % QLatin1String("hint-top-inset"))) {
+        frame->insetTopMargin = q->elementSize(frame->prefix % QLatin1String("hint-top-inset")).height();
+    } else {
+        frame->insetTopMargin = -1;
+    }
+
     frame->fixedLeftWidth = q->elementSize(frame->prefix % QLatin1String("left")).width();
 
     if (q->hasElement(frame->prefix % QLatin1String("hint-left-margin"))) {
@@ -784,6 +833,12 @@ void FrameSvgPrivate::updateSizes(FrameData *frame) const
         frame->leftMargin = frame->leftWidth = 0;
     }
 
+    if (q->hasElement(frame->prefix % QLatin1String("hint-left-inset"))) {
+        frame->insetLeftMargin = q->elementSize(frame->prefix % QLatin1String("hint-left-inset")).width();
+    } else {
+        frame->insetLeftMargin = -1;
+    }
+
     frame->fixedRightWidth = q->elementSize(frame->prefix % QLatin1String("right")).width();
 
     if (q->hasElement(frame->prefix % QLatin1String("hint-right-margin"))) {
@@ -799,6 +854,12 @@ void FrameSvgPrivate::updateSizes(FrameData *frame) const
         frame->rightMargin = frame->rightWidth = 0;
     }
 
+    if (q->hasElement(frame->prefix % QLatin1String("hint-right-inset"))) {
+        frame->insetRightMargin = q->elementSize(frame->prefix % QLatin1String("hint-right-inset")).width();
+    } else {
+        frame->insetRightMargin = -1;
+    }
+
     frame->fixedBottomHeight = q->elementSize(frame->prefix % QLatin1String("bottom")).height();
 
     if (q->hasElement(frame->prefix % QLatin1String("hint-bottom-margin"))) {
@@ -814,6 +875,12 @@ void FrameSvgPrivate::updateSizes(FrameData *frame) const
         frame->bottomMargin = frame->bottomHeight = 0;
     }
 
+    if (q->hasElement(frame->prefix % QLatin1String("hint-bottom-inset"))) {
+        frame->insetBottomMargin = q->elementSize(frame->prefix % QLatin1String("hint-bottom-inset")).height();
+    } else {
+        frame->insetBottomMargin = -1;
+    }
+
     frame->composeOverBorder = (q->hasElement(frame->prefix % QLatin1String("hint-compose-over-border")) &&
                                 q->hasElement(QLatin1String("mask-") % frame->prefix % QLatin1String("center")));
 
@@ -875,7 +942,7 @@ void FrameSvg::setRepaintBlocked(bool blocked)
     d->repaintBlocked = blocked;
 
     if (!blocked) {
-        d->updateFrameData();
+        d->updateFrameData(Svg::d->lastModified);
     }
 }
 
