@@ -162,6 +162,25 @@ public:
      */
     Q_INVOKABLE void getFixedMargins(qreal &left, qreal &top, qreal &right, qreal &bottom) const;
 
+    /**
+     * Returns the insets margin size given the margin edge we want.
+     * @param edge the margin edge we want, top, bottom, left or right
+     * @return the margin size
+     * @since 5.77
+     */
+    Q_INVOKABLE qreal insetSize(const Plasma::Types::MarginEdge edge) const;
+
+    /**
+     * Convenience method that extracts the size of the four inset margins
+     * in the four output parameters
+     * @param left left margin size
+     * @param top top margin size
+     * @param right right margin size
+     * @param bottom bottom margin size
+     * @since 5.77
+     */
+    Q_INVOKABLE void getInset(qreal &left, qreal &top, qreal &right, qreal &bottom) const;
+
     /**
      * @return the rectangle of the center element, taking the margins into account.
      */
