@@ -55,11 +55,13 @@ public:
 	 * This enumeration lists the various hints that an applet can pass to its
 	 * constraint regarding the way that it is represented
      */
-    enum ConstraintHints {
+    enum ConstraintHints { //TODO KF6 ConstraintHints -> ConstraintHint
 		NoHint = 0,
-		CanFillArea = 1 /**< The applet can fill the area and ignore constraint margins*/
+		CanFillArea = 1, /**< The CompactRepresentation can fill the area and ignore constraint margins*/ // (TODO: KF6 CanFillArea -> CompactRepresentationFillArea)
+		MarginAreasSeparator = CanFillArea | 2, /**< The applet acts as a separator between the standard and slim panel margin areas*/
     };
-    Q_ENUM(ConstraintHints)
+    Q_DECLARE_FLAGS(ConstraintFlags, ConstraintHints)
+    Q_FLAG(ConstraintFlags)
 
     /**
      * The FormFactor enumeration describes how a Plasma::Applet should arrange
@@ -86,6 +88,20 @@ public:
     };
     Q_ENUM(FormFactor)
 
+    /**
+     * Display hints that come from the containment that suggest the applet how to look and behave.
+     * @since 5.77
+     */
+    enum ContainmentDisplayHint {
+        NoContainmentDisplayHint = 0,
+        ContainmentDrawsPlasmoidHeading = 1, /**< The containment will draw an titlebar-looking header for the applets, so the applets shouldn't attempt to paint a similar thing **/
+        ContainmentForcesSquarePlasmoids = 2, /**< The containment will force every plasmoid to be constrained in a square icon (An example is the System Tray)**/
+        DesktopFullyCovered = 4 /**< The desktop area for the contaiment's screen is not visible at all, for instance a window has been maximized on top of it */
+    };
+    Q_ENUM(ContainmentDisplayHint)
+    Q_DECLARE_FLAGS(ContainmentDisplayHints, ContainmentDisplayHint)
+    Q_FLAG(ContainmentDisplayHints)
+
     /**
      * This enumeration describes the type of the Containment.
      * DesktopContainments represent main containments that will own a screen in a mutually exclusive fashion,
@@ -95,6 +111,7 @@ public:
         NoContainmentType = -1,  /**< @internal */
         DesktopContainment = 0,  /**< A desktop containment */
         PanelContainment,        /**< A desktop panel */
+
         CustomContainment = 127, /**< A containment that is neither a desktop nor a panel
                                 but something application specific */
         CustomPanelContainment = 128, /**< A customized desktop panel */
@@ -307,7 +324,9 @@ PLASMA_EXPORT Types::Direction locationToDirection(Types::Location location);
  **/
 PLASMA_EXPORT Types::Direction locationToInverseDirection(Types::Location location);
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Types::ContainmentDisplayHints)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Types::Constraints)
+Q_DECLARE_OPERATORS_FOR_FLAGS(Types::ConstraintFlags)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Types::Flip)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Types::ComponentTypes)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Types::BackgroundFlags)
