@@ -1,21 +1,8 @@
 /*
- *   Copyright 2010 Ryan Rix <ry@n.rix.si>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2010 Ryan Rix <ry@n.rix.si>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "pluginloader.h"
 
@@ -23,11 +10,12 @@
 #include <QStandardPaths>
 
 #include <QDebug>
-#include <kservice.h>
-#include <kservicetypetrader.h>
-#include <kplugintrader.h>
+#include <KService>
+#include <KServiceTypeTrader>
+#include <KPluginTrader>
 #include <KPluginLoader>
 #include <kpackage/packageloader.h>
+#include <kdeclarative/kdeclarative.h>
 
 #include "config-plasma.h"
 
@@ -133,9 +121,8 @@ PluginLoader::PluginLoader()
 
 PluginLoader::~PluginLoader()
 {
-    typedef QPointer<PackageStructure> pswp;
-    foreach (pswp wp, d->structures) {
-        delete wp.data();
+    for (const auto &wp : qAsConst(d->structures)) {
+        delete wp;
     }
     delete d;
 }
@@ -208,7 +195,6 @@ Applet *PluginLoader::loadApplet(const QString &name, uint appletId, const QVari
         return applet;
     }
 
-
     if (!applet) {
         //qCDebug(LOG_PLASMA) << name << "not a C++ applet: Falling back to an empty one";
 
@@ -272,22 +258,24 @@ QStringList PluginLoader::listAllEngines(const QString &parentApp)
         plugins = KPluginLoader::findPlugins(PluginLoaderPrivate::s_dataEnginePluginDir, filter);
     }
 
-    foreach (auto& plugin, plugins) {
+    for (auto& plugin : qAsConst(plugins)) {
         engines << plugin.pluginId();
     }
 
     const QList<KPluginMetaData> packagePlugins = KPackage::PackageLoader::self()->listPackages(QStringLiteral("Plasma/DataEngine"));
-    for (auto& plugin : packagePlugins) {
+    for (const auto &plugin : packagePlugins) {
         engines << plugin.pluginId();
     }
 
     return engines;
 }
 
+#if PLASMA_BUILD_DEPRECATED_SINCE(5, 77)
 KPluginInfo::List PluginLoader::listEngineInfo(const QString &parentApp)
 {
     return PluginLoader::self()->listDataEngineInfo(parentApp);
 }
+#endif
 
 KPluginInfo::List PluginLoader::listEngineInfoByCategory(const QString &category, const QString &parentApp)
 {
@@ -447,14 +435,29 @@ Package PluginLoader::loadPackage(const QString &packageFormat, const QString &s
         structure->d->internalStructure = internalStructure;
     //fallback to old structures
     } else {
-        const QString constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(packageFormat);
-        structure = KPluginTrader::createInstanceFromQuery<Plasma::PackageStructure>(PluginLoaderPrivate::s_packageStructurePluginDir, QStringLiteral("Plasma/PackageStructure"), constraint, nullptr);
-        if (structure) {
-            structure->d->internalStructure = new PackageStructureWrapper(structure);
+
+        auto filter = [packageFormat](const KPluginMetaData &md) -> bool
+        {
+            return md.value(QStringLiteral("X-KDE-PluginInfo-Name")) == packageFormat;
+        };
+
+        const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(PluginLoaderPrivate::s_packageStructurePluginDir, filter);
+
+        if (!plugins.isEmpty()) {
+            KPluginLoader loader(plugins.first().fileName());
+            KPluginFactory* factory = loader.factory();
+            if (!factory) {
+                qWarning() << "Error loading plugin:" << loader.errorString();
+            } else {
+                structure = factory->create<Plasma::PackageStructure>();
+            }
+
+            if (structure) {
+                structure->d->internalStructure = new PackageStructureWrapper(structure);
+            }
         }
     }
 
-
     if (structure) {
         d->structures.insert(hashkey, structure);
         return Package(structure);
@@ -469,22 +472,57 @@ Package PluginLoader::loadPackage(const QString &packageFormat, const QString &s
 
 QList<KPluginMetaData> PluginLoader::listAppletMetaData(const QString &category, const QString &parentApp)
 {
+    auto platforms = KDeclarative::KDeclarative::runtimePlatform();
+    // For now desktop always lists everything
+    if (platforms.contains(QStringLiteral("desktop"))) {
+        platforms.clear();
+    }
+
     //FIXME: this assumes we are always use packages.. no pure c++
     std::function<bool(const KPluginMetaData&)> filter;
     if (category.isEmpty()) { //use all but the excluded categories
         KConfigGroup group(KSharedConfig::openConfig(), "General");
         QStringList excluded = group.readEntry("ExcludeCategories", QStringList());
 
-        filter = [excluded, parentApp](const KPluginMetaData &md) -> bool
+        filter = [excluded, parentApp, platforms](const KPluginMetaData &md) -> bool
         {
+            if (!platforms.isEmpty() && !md.formFactors().isEmpty()) {
+                bool found = false;
+                for (const auto &plat : platforms) {
+                    if (md.formFactors().contains(plat)) {
+                        found = true;
+                        break;
+                    }
+                }
+
+                if (!found) {
+                    return false;
+                }
+            }
+
             const QString pa = md.value(QStringLiteral("X-KDE-ParentApp"));
             return (parentApp.isEmpty() || pa == parentApp) && !excluded.contains(md.category());
         };
     } else { //specific category (this could be an excluded one - is that bad?)
 
-        filter = [category, parentApp](const KPluginMetaData &md) -> bool
+        filter = [category, parentApp, platforms](const KPluginMetaData &md) -> bool
         {
+            if (!platforms.isEmpty() && !md.formFactors().isEmpty()) {
+                bool found = false;
+                for (const auto &plat : platforms) {
+                    if (md.formFactors().contains(plat)) {
+                        found = true;
+                        break;
+                    }
+                }
+
+                if (!found) {
+                    return false;
+                }
+            }
+
             const QString pa = md.value(QStringLiteral("X-KDE-ParentApp"));
+
             if (category == QLatin1String("Miscellaneous")) {
                 return (parentApp.isEmpty() || pa == parentApp) && (md.category() == category || md.category().isEmpty());
             } else {
@@ -502,13 +540,18 @@ QList<KPluginMetaData> PluginLoader::listAppletMetaData(const QString &category,
 
 KPluginInfo::List PluginLoader::listAppletInfo(const QString &category, const QString &parentApp)
 {
-    KPluginInfo::List list;
     const auto plugins = listAppletMetaData(category, parentApp);
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
+    KPluginInfo::List list;
     //NOTE: it still produces kplugininfos from KServices because some user code expects
     //info.service() to be valid and would crash otherwise
-    foreach (auto& md, plugins) {
+    for (const auto &md : plugins) {
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
         auto pi = md.metaDataFileName().endsWith(QLatin1String(".json")) ? KPluginInfo(md) : KPluginInfo(KService::serviceByStorageId(md.metaDataFileName()));
+QT_WARNING_POP
         if (!pi.isValid()) {
             qCWarning(LOG_PLASMA) << "Could not load plugin info for plugin :" << md.pluginId() << "skipping plugin";
             continue;
@@ -516,6 +559,9 @@ KPluginInfo::List PluginLoader::listAppletInfo(const QString &category, const QS
         list << pi;
     }
     return list;
+#else
+    return KPluginInfo::fromMetaData(plugins.toVector());
+#endif
 }
 
 QList<KPluginMetaData> PluginLoader::listAppletMetaDataForMimeType(const QString &mimeType)
@@ -548,9 +594,9 @@ QList<KPluginMetaData> PluginLoader::listAppletMetaDataForUrl(const QUrl &url)
     const QList<KPluginMetaData> allApplets = KPackage::PackageLoader::self()->findPackages(QStringLiteral("Plasma/Applet"), QString(), filter);
 
     QList<KPluginMetaData> filtered;
-    foreach (const KPluginMetaData &md, allApplets) {
-        QStringList urlPatterns = KPluginMetaData::readStringList(md.rawData(), QStringLiteral("X-Plasma-DropUrlPatterns"));
-        foreach (const QString &glob, urlPatterns) {
+    for (const KPluginMetaData &md : allApplets) {
+        const QStringList urlPatterns = KPluginMetaData::readStringList(md.rawData(), QStringLiteral("X-Plasma-DropUrlPatterns"));
+        for (const QString &glob : urlPatterns) {
             QRegExp rx(glob);
             rx.setPatternSyntax(QRegExp::Wildcard);
             if (rx.exactMatch(url.toString())) {
@@ -585,7 +631,7 @@ QStringList PluginLoader::listAppletCategories(const QString &parentApp, bool vi
 
 
     QStringList categories;
-    foreach (auto& plugin, allApplets) {
+    for (auto &plugin : allApplets) {
         if (plugin.category().isEmpty()) {
             if (!categories.contains(i18nc("misc category", "Miscellaneous"))) {
                 categories << i18nc("misc category", "Miscellaneous");
@@ -600,12 +646,12 @@ QStringList PluginLoader::listAppletCategories(const QString &parentApp, bool vi
 
 void PluginLoader::setCustomAppletCategories(const QStringList &categories)
 {
-    PluginLoaderPrivate::s_customCategories = QSet<QString>::fromList(categories);
+    PluginLoaderPrivate::s_customCategories = QSet<QString>(categories.begin(), categories.end());
 }
 
 QStringList PluginLoader::customAppletCategories() const
 {
-    return PluginLoaderPrivate::s_customCategories.toList();
+    return PluginLoaderPrivate::s_customCategories.values();
 }
 
 QString PluginLoader::appletCategory(const QString &appletName)
@@ -669,65 +715,88 @@ KPluginInfo::List PluginLoader::listContainmentsForMimeType(const QString &mimeT
 
 QStringList PluginLoader::listContainmentTypes()
 {
-    KPluginInfo::List containmentInfos = listContainments();
+    const KPluginInfo::List containmentInfos = listContainments();
     QSet<QString> types;
 
-    foreach (const KPluginInfo &containmentInfo, containmentInfos) {
-        QStringList theseTypes = containmentInfo.service()->property(QStringLiteral("X-Plasma-ContainmentType")).toStringList();
-        foreach (const QString &type, theseTypes) {
+    for (const KPluginInfo &containmentInfo : containmentInfos) {
+        const QStringList theseTypes = containmentInfo.property(QStringLiteral("X-Plasma-ContainmentType")).toStringList();
+        for (const QString &type : theseTypes) {
             types.insert(type);
         }
     }
 
-    return types.toList();
+    return types.values();
 }
 
+#if PLASMA_BUILD_DEPRECATED_SINCE(5, 77)
 KPluginInfo::List PluginLoader::listDataEngineInfo(const QString &parentApp)
 {
-    KPluginInfo::List list;
+    return KPluginInfo::fromMetaData(listDataEngineMetaData(parentApp));
+}
+#endif
 
-    if (!d->isDefaultLoader && (parentApp.isEmpty() || parentApp == QCoreApplication::instance()->applicationName())) {
-        list = internalDataEngineInfo();
+QVector<KPluginMetaData> PluginLoader::listDataEngineMetaData(const QString &parentApp)
+{
+    auto filter = [&parentApp](const KPluginMetaData &md) -> bool
+    {
+        return md.value(QStringLiteral("X-KDE-ParentApp")) == parentApp;
+    };
+
+    QVector<KPluginMetaData> plugins;
+    if (parentApp.isEmpty()) {
+        plugins = KPluginLoader::findPlugins(PluginLoaderPrivate::s_dataEnginePluginDir);
+    } else {
+        plugins = KPluginLoader::findPlugins(PluginLoaderPrivate::s_dataEnginePluginDir, filter);
     }
 
-    QString constraint;
-    if (!parentApp.isEmpty()) {
-        constraint = QLatin1String("[X-KDE-ParentApp] == '") + parentApp + QLatin1Char('\'');
-    }
-
-    list.append(KPluginTrader::self()->query(PluginLoaderPrivate::s_dataEnginePluginDir, QStringLiteral("Plasma/DataEngine"), constraint));
-    return list;
+    return plugins;
 }
 
+#if PLASMA_BUILD_DEPRECATED_SINCE(5, 77)
 KPluginInfo::List PluginLoader::listContainmentActionsInfo(const QString &parentApp)
 {
-    KPluginInfo::List list;
+    return KPluginInfo::fromMetaData(listContainmentActionsMetaData(parentApp));
+}
+#endif
 
-    if (!d->isDefaultLoader && (parentApp.isEmpty() || parentApp == QCoreApplication::instance()->applicationName())) {
-        list = internalContainmentActionsInfo();
+QVector<KPluginMetaData> PluginLoader::listContainmentActionsMetaData(const QString &parentApp)
+{
+    auto filter = [&parentApp](const KPluginMetaData &md) -> bool
+    {
+        return md.value(QStringLiteral("X-KDE-ParentApp")) == parentApp;
+    };
+
+    QVector<KPluginMetaData> plugins;
+    if (parentApp.isEmpty()) {
+        plugins = KPluginLoader::findPlugins(PluginLoaderPrivate::s_containmentActionsPluginDir);
+    } else {
+        plugins = KPluginLoader::findPlugins(PluginLoaderPrivate::s_containmentActionsPluginDir, filter);
     }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
+    //FIXME: this is only for backwards compatibility, but probably will have to stay
+    //for the time being
+    QSet<QString> knownPlugins;
+    for (const KPluginMetaData &p : qAsConst(plugins)) {
+        knownPlugins.insert(p.pluginId());
+    }
     QString constraint;
     if (!parentApp.isEmpty()) {
         constraint = QLatin1String("[X-KDE-ParentApp] == '") + parentApp + QLatin1Char('\'');
     }
-
-    list.append(KPluginTrader::self()->query(PluginLoaderPrivate::s_containmentActionsPluginDir, QStringLiteral("Plasma/ContainmentActions"), constraint));
-
-    QSet<QString> knownPlugins;
-    foreach (const KPluginInfo &p, list) {
-        knownPlugins.insert(p.pluginName());
-    }
-
-    //FIXME: this is only for backwards compatibility, but probably will have to stay
-    //for the time being
-    KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/ContainmentActions"), constraint);
-    foreach (KService::Ptr s, offers) {
+    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/ContainmentActions"), constraint);
+    for (KService::Ptr s : offers) {
         if (!knownPlugins.contains(s->pluginKeyword())) {
-            list.append(KPluginInfo(s));
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+            plugins.append(KPluginInfo(s).toMetaData());
+QT_WARNING_POP
         }
     }
-    return list;
+#endif
+
+    return plugins;
 }
 
 Applet *PluginLoader::internalLoadApplet(const QString &name, uint appletId, const QVariantList &args)
@@ -795,14 +864,14 @@ static KPluginInfo::List standardInternalInfo(const QString &type, const QString
                         QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/internal/") + type + QLatin1String("/*.desktop"),
                         QStandardPaths::LocateFile);
 
-    KPluginInfo::List allInfo = KPluginInfo::fromFiles(files);
+    const KPluginInfo::List allInfo = KPluginInfo::fromFiles(files);
 
     if (category.isEmpty() || allInfo.isEmpty()) {
         return allInfo;
     }
 
     KPluginInfo::List matchingInfo;
-    foreach (const KPluginInfo &info, allInfo) {
+    for (const KPluginInfo &info : allInfo) {
         if (info.category().compare(category, Qt::CaseInsensitive) == 0) {
             matchingInfo << info;
         }
@@ -861,7 +930,7 @@ QVector<KPluginMetaData> PluginLoaderPrivate::Cache::findPluginsById(const QStri
         auto insertIntoCache = [this](const QString &pluginPath) {
             KPluginMetaData metadata(pluginPath);
             if (!metadata.isValid()) {
-                qCWarning(LOG_PLASMA) << "invalid metadata" << pluginPath;
+                qCDebug(LOG_PLASMA) << "invalid metadata" << pluginPath;
                 return;
             }
 
