@@ -1,24 +1,10 @@
-/////////////////////////////////////////////////////////////////////////
-// storage.cpp                                                         //
-//                                                                     //
-// Copyright (C)  2010  Brian Pritchett <batenkaitos@gmail.com>        //
-// Copyright (C)  2010  Marco Martin <mart@kde.org>                    //
-//                                                                     //
-// This library is free software; you can redistribute it and/or       //
-// modify it under the terms of the GNU Lesser General Public          //
-// License as published by the Free Software Foundation; either        //
-// version 2.1 of the License, or (at your option) any later version.  //
-//                                                                     //
-// This library is distributed in the hope that it will be useful,     //
-// but WITHOUT ANY WARRANTY; without even the implied warranty of      //
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   //
-// Lesser General Public License for more details.                     //
-//                                                                     //
-// You should have received a copy of the GNU Lesser General Public    //
-// License along with this library; if not, write to the Free Software //
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA       //
-// 02110-1301  USA                                                     //
-/////////////////////////////////////////////////////////////////////////
+/*
+    storage.cpp
+    SPDX-FileCopyrightText: 2010 Brian Pritchett <batenkaitos@gmail.com>
+    SPDX-FileCopyrightText: 2010 Marco Martin <mart@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "private/storage_p.h"
 
@@ -46,9 +32,9 @@ StorageJob::StorageJob(const QString &destination,
       m_clientName(destination)
 {
     Plasma::StorageThread::self()->start();
-    connect(Plasma::StorageThread::self(), SIGNAL(newResult(StorageJob*,QVariant)), this, SLOT(resultSlot(StorageJob*,QVariant)));
+    connect(Plasma::StorageThread::self(), &Plasma::StorageThread::newResult, this, &StorageJob::resultSlot);
     qRegisterMetaType<StorageJob *>();
-    qRegisterMetaType<QWeakPointer<StorageJob> >();
+    qRegisterMetaType<QPointer<StorageJob> >();
 }
 
 StorageJob::~StorageJob()
@@ -80,15 +66,15 @@ void StorageJob::start()
         valueGroup = QStringLiteral("default");
     }
 
-    QWeakPointer<StorageJob> me(this);
+    QPointer<StorageJob> me(this);
     if (operationName() == QLatin1String("save")) {
-        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "save", Qt::QueuedConnection, Q_ARG(QWeakPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
+        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "save", Qt::QueuedConnection, Q_ARG(QPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
     } else if (operationName() == QLatin1String("retrieve")) {
-        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "retrieve", Qt::QueuedConnection, Q_ARG(QWeakPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
+        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "retrieve", Qt::QueuedConnection, Q_ARG(QPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
     } else if (operationName() == QLatin1String("delete")) {
-        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "deleteEntry", Qt::QueuedConnection, Q_ARG(QWeakPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
+        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "deleteEntry", Qt::QueuedConnection, Q_ARG(QPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
     } else if (operationName() == QLatin1String("expire")) {
-        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "expire", Qt::QueuedConnection, Q_ARG(QWeakPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
+        QMetaObject::invokeMethod(Plasma::StorageThread::self(), "expire", Qt::QueuedConnection, Q_ARG(QPointer<StorageJob>, me), Q_ARG(QVariantMap, params));
     } else {
         setError(true);
         setResult(false);
@@ -131,13 +117,13 @@ Storage::Storage(QObject *parent)
 
         Plasma::DataEngine *engine = qobject_cast<Plasma::DataEngine *>(parentObject);
         if (engine) {
-            m_clientName = engine->pluginInfo().pluginName();
+            m_clientName = engine->metadata().pluginId();
             break;
         }
     }
 
-    m_clientName = m_clientName.replace(QLatin1Char('.'), QLatin1Char('_'));
-    m_clientName = m_clientName.replace(QLatin1Char('-'), QLatin1Char('_'));
+    m_clientName.replace(QLatin1Char('.'), QLatin1Char('_'));
+    m_clientName.replace(QLatin1Char('-'), QLatin1Char('_'));
 
     setName(QStringLiteral("storage"));
 }
