@@ -8,6 +8,7 @@
 #include "theme_p.h"
 #include "framesvg.h"
 #include "framesvg_p.h"
+#include "svg_p.h"
 #include "debug_p.h"
 
 #include <QGuiApplication>
@@ -75,12 +76,6 @@ ThemePrivate::ThemePrivate(QObject *parent)
     pixmapSaveTimer->setInterval(600);
     QObject::connect(pixmapSaveTimer, &QTimer::timeout, this, &ThemePrivate::scheduledCacheUpdate);
 
-    rectSaveTimer = new QTimer(this);
-    rectSaveTimer->setSingleShot(true);
-    //2 minutes
-    rectSaveTimer->setInterval(2 * 60 * 1000);
-    QObject::connect(rectSaveTimer, &QTimer::timeout, this, &ThemePrivate::saveSvgElementsCache);
-
     updateNotificationTimer = new QTimer(this);
     updateNotificationTimer->setSingleShot(true);
     updateNotificationTimer->setInterval(100);
@@ -121,7 +116,6 @@ ThemePrivate::ThemePrivate(QObject *parent)
 
 ThemePrivate::~ThemePrivate()
 {
-    saveSvgElementsCache();
     FrameSvgPrivate::s_sharedFrames.remove(this);
     delete pixmapCache;
 }
@@ -239,43 +233,24 @@ bool ThemePrivate::useCache()
 
         ThemeConfig config;
         pixmapCache = new KImageCache(cacheFile, config.themeCacheKb() * 1024);
+        pixmapCache->setEvictionPolicy(KSharedDataCache::EvictLeastRecentlyUsed);
 
         if (cachesTooOld) {
             discardCache(PixmapCache | SvgElementsCache);
         }
     }
 
-    if (cacheTheme && !svgElementsCache) {
-        const QString svgElementsFileNameBase = QLatin1String("plasma-svgelements-") + themeName;
-        QString svgElementsFileName = svgElementsFileNameBase;
-        if (!themeVersion.isEmpty()) {
-            svgElementsFileName += QLatin1String("_v") + themeVersion;
-        }
-
-        // now we check for (and remove) old caches
-        QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation));
-        cacheDir.setNameFilters(QStringList({svgElementsFileNameBase + QLatin1Char('*')}));
-
-        const auto files = cacheDir.entryInfoList();
-        for (const QFileInfo &file : files) {
-            if (!file.absoluteFilePath().endsWith(svgElementsFileName)) {
-                QFile::remove(file.absoluteFilePath());
-            }
-        }
-
-        const QString svgElementsFile = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1Char('/') + svgElementsFileName;
-        svgElementsCache = KSharedConfig::openConfig(svgElementsFile, KConfig::SimpleConfig);
+    if (cacheTheme) {
         QString currentIconThemePath;
         const auto *iconTheme = KIconLoader::global()->theme();
         if (iconTheme) {
             currentIconThemePath = iconTheme->dir();
         }
-        KConfigGroup globalGroup(svgElementsCache, QLatin1String("Global"));
-        const QString oldIconThemePath = globalGroup.readEntry("currentIconThemePath", QString());
+
+        const QString oldIconThemePath = SvgRectsCache::instance()->iconThemePath();
         if (oldIconThemePath != currentIconThemePath) {
             discardCache(PixmapCache | SvgElementsCache);
-            globalGroup.writeEntry("currentIconThemePath", currentIconThemePath);
-            svgElementsCache = KSharedConfig::openConfig(svgElementsFile, KConfig::SimpleConfig);
+            SvgRectsCache::instance()->setIconThemePath(currentIconThemePath);
         }
     }
 
@@ -357,9 +332,6 @@ void ThemePrivate::discardCache(CacheTypes caches)
 
     if (caches & SvgElementsCache) {
         discoveries.clear();
-        invalidElements.clear();
-
-        svgElementsCache = nullptr;
     }
 }
 
@@ -657,21 +629,6 @@ void ThemePrivate::settingsChanged(bool emitChanges)
     setThemeName(cg.readEntry("name", ThemePrivate::defaultTheme), false, emitChanges);
 }
 
-void ThemePrivate::saveSvgElementsCache()
-{
-    if (svgElementsCache) {
-        QHashIterator<QString, QSet<QString> > it(invalidElements);
-        while (it.hasNext()) {
-            it.next();
-            KConfigGroup imageGroup(svgElementsCache, it.key());
-            imageGroup.writeEntry("invalidElements", it.value().values()); //FIXME: add QSet support to KConfig
-        }
-
-        //Pretty drastic, but this is executed only very rarely
-        svgElementsCache->sync();
-    }
-}
-
 QColor ThemePrivate::color(Theme::ColorRole role, Theme::ColorGroup group) const
 {
     const KColorScheme *scheme = nullptr;
