@@ -1,22 +1,9 @@
 /*
- *   Copyright 2006-2007 Aaron Seigo <aseigo@kde.org>
- *   Copyright 2013 Marco Martin <mart@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2013 Marco Martin <mart@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_THEME_P_H
 #define PLASMA_THEME_P_H
@@ -26,10 +13,11 @@
 #include <QHash>
 
 #include <QDebug>
-#include <kcolorscheme.h>
-#include <kimagecache.h>
-#include <kshareddatacache.h>
-#include <kwindowsystem.h>
+#include <KColorScheme>
+#include <KImageCache>
+#include <KSharedDataCache>
+#include <KWindowSystem>
+#include <KPluginMetaData>
 #include <QTimer>
 
 #include <config-plasma.h>
@@ -58,13 +46,13 @@ enum CacheType {
 Q_DECLARE_FLAGS(CacheTypes, CacheType)
 Q_DECLARE_OPERATORS_FOR_FLAGS(CacheTypes)
 
-class ThemePrivate : public QObject
+class ThemePrivate : public QObject, public QSharedData
 {
     Q_OBJECT
 
 public:
     explicit ThemePrivate(QObject *parent = nullptr);
-    ~ThemePrivate() Q_DECL_OVERRIDE;
+    ~ThemePrivate() override;
 
     KConfigGroup &config();
 
@@ -76,6 +64,7 @@ public:
     void setThemeName(const QString &themeName, bool writeSettings, bool emitChanged);
     void processWallpaperSettings(KConfigBase *metadata);
     void processContrastSettings(KConfigBase *metadata);
+    void processBlurBehindSettings(KConfigBase *metadata);
 
     const QString processStyleSheet(const QString &css, Plasma::Svg::Status status);
     const QString svgStyleSheet(Plasma::Theme::ColorGroup group, Plasma::Svg::Status status);
@@ -89,7 +78,6 @@ public Q_SLOTS:
     void onAppExitCleanup();
     void notifyOfChanged();
     void settingsChanged(bool emitChanges);
-    void saveSvgElementsCache();
 
 Q_SIGNALS:
     void themeChanged();
@@ -106,12 +94,10 @@ public:
 #endif
 //Ref counting of ThemePrivate instances
     static ThemePrivate *globalTheme;
-    static QAtomicInt globalThemeRefCount;
     static QHash<QString, ThemePrivate *> themes;
-    static QHash<QString, QAtomicInt> themesRefCount;
 
     QString themeName;
-    KPluginInfo pluginInfo;
+    KPluginMetaData pluginMetaData;
     QList<QString> fallbackThemes;
     KSharedConfigPtr colors;
     KColorScheme colorScheme;
@@ -119,16 +105,17 @@ public:
     KColorScheme buttonColorScheme;
     KColorScheme viewColorScheme;
     KColorScheme complementaryColorScheme;
-    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
+    KColorScheme headerColorScheme;
+    KColorScheme tooltipColorScheme;
+    QPalette palette;
+    bool eventFilter(QObject *watched, QEvent *event) override;
     KConfigGroup cfg;
     QString defaultWallpaperTheme;
     QString defaultWallpaperSuffix;
     int defaultWallpaperWidth;
     int defaultWallpaperHeight;
     KImageCache *pixmapCache;
-    KSharedConfigPtr svgElementsCache;
     QString cachedDefaultStyleSheet;
-    QHash<QString, QSet<QString> > invalidElements;
     QHash<QString, QPixmap> pixmapsToCache;
     QHash<QString, QString> keysToCache;
     QHash<QString, QString> idsToCache;
@@ -136,7 +123,6 @@ public:
     QHash<Theme::ColorGroup, QString> cachedSelectedSvgStyleSheets;
     QHash<QString, QString> discoveries;
     QTimer *pixmapSaveTimer;
-    QTimer *rectSaveTimer;
     QTimer *updateNotificationTimer;
     unsigned cacheSize;
     CacheTypes cachesToDiscard;
@@ -156,6 +142,7 @@ public:
     qreal backgroundIntensity;
     qreal backgroundSaturation;
     bool backgroundContrastEnabled;
+    bool blurBehindEnabled;
 
     //Version number of Plasma the Theme has been designed for
     int apiMajor;
