@@ -1,28 +1,16 @@
 /*
- *   Copyright 2006-2007 Aaron Seigo <aseigo@kde.org>
- *   Copyright 2008-2010 Marco Martin <notmart@gmail.com>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2008-2010 Marco Martin <notmart@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "svg.h"
 #include "private/svg_p.h"
 #include "private/theme_p.h"
 
 #include <cmath>
+#include <array>
 
 #include <QCoreApplication>
 #include <QDir>
@@ -47,9 +35,36 @@
 #include "theme.h"
 #include "debug_p.h"
 
+uint qHash(const Plasma::SvgPrivate::CacheId &id, uint seed)
+{
+    std::array<uint, 10> parts = {
+        ::qHash(id.width),
+        ::qHash(id.height),
+        ::qHash(id.elementName),
+        ::qHash(id.filePath),
+        ::qHash(id.status),
+        ::qHash(id.devicePixelRatio),
+        ::qHash(id.scaleFactor),
+        ::qHash(id.colorGroup),
+        ::qHash(id.extraFlags),
+        ::qHash(id.lastModified)
+    };
+    return qHashRange(parts.begin(), parts.end(), seed);
+}
+
+
 namespace Plasma
 {
 
+class SvgRectsCacheSingleton
+{
+public:
+    SvgRectsCache self;
+};
+
+Q_GLOBAL_STATIC(SvgRectsCacheSingleton, privateSvgRectsCacheSelf)
+
+const uint SvgRectsCache::s_seed = 0x9e3779b9;
 
 SharedSvgRenderer::SharedSvgRenderer(QObject *parent)
     : QSvgRenderer(parent)
@@ -136,9 +151,217 @@ bool SharedSvgRenderer::load(
     return true;
 }
 
-#define QLSEP QLatin1Char('_')
-#define CACHE_ID_WITH_SIZE(size, id, status, devicePixelRatio) QString::number(int(size.width())) % QLSEP % QString::number(int(size.height())) % QLSEP % id % QLSEP % QString::number(status) % QLSEP % QString::number(int(devicePixelRatio))
-#define CACHE_ID_NATURAL_SIZE(id, status, devicePixelRatio) QLatin1String("Natural") % QLSEP % id % QLSEP % QString::number(status) % QLSEP % QString::number(int(devicePixelRatio))
+SvgRectsCache::SvgRectsCache(QObject *parent)
+    : QObject(parent)
+{
+    const QString svgElementsFile = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1Char('/') + QStringLiteral("plasma-svgelements");
+    m_svgElementsCache = KSharedConfig::openConfig(svgElementsFile, KConfig::SimpleConfig);
+
+    m_configSyncTimer = new QTimer(this);
+    m_configSyncTimer->setSingleShot(true);
+    m_configSyncTimer->setInterval(5000);
+    connect(m_configSyncTimer, &QTimer::timeout,
+            this, [this]() {
+                m_svgElementsCache->sync();
+            });
+}
+
+SvgRectsCache *SvgRectsCache::instance()
+{
+    return &privateSvgRectsCacheSelf()->self;
+}
+
+void SvgRectsCache::insert(Plasma::SvgPrivate::CacheId cacheId, const QRectF &rect, unsigned int lastModified)
+{
+    insert(qHash(cacheId, SvgRectsCache::s_seed), cacheId.filePath, rect, lastModified);
+}
+
+void SvgRectsCache::insert(uint id, const QString &filePath, const QRectF &rect, unsigned int lastModified)
+{
+    if (m_localRectCache.contains(id)) {
+        return;
+    }
+
+    m_localRectCache.insert(id, rect);
+
+    KConfigGroup imageGroup(m_svgElementsCache, filePath);
+    imageGroup.writeEntry("LastModified", lastModified);
+    if (rect.isValid()) {
+        imageGroup.writeEntry(QString::number(id), rect);
+    } else {
+        m_invalidElements[filePath] << id;
+        imageGroup.writeEntry("Invalidelements", m_invalidElements[filePath].values());
+    }
+    QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+}
+
+bool SvgRectsCache::findElementRect(Plasma::SvgPrivate::CacheId cacheId, QRectF &rect)
+{
+    return findElementRect(qHash(cacheId, SvgRectsCache::s_seed), cacheId.filePath, rect);
+}
+
+bool SvgRectsCache::findElementRect(uint id, const QString &filePath, QRectF &rect)
+{
+    auto it = m_localRectCache.find(id);
+
+    if (it == m_localRectCache.end()) {
+        if (m_invalidElements.contains(filePath) && m_invalidElements[filePath].contains(id)) {
+            rect = QRectF();
+            return true;
+        }
+        return false;
+    }
+
+    rect = *it;
+
+    return true;
+}
+
+void SvgRectsCache::loadImageFromCache(const QString &path, uint lastModified)
+{
+    if (path.isEmpty()) {
+        return;
+    }
+
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+
+    unsigned int savedTime = imageGroup.readEntry("LastModified", 0);
+
+    if (lastModified > savedTime) {
+        imageGroup.deleteGroup();
+        QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+        return;
+    }
+
+    auto list = imageGroup.readEntry("Invalidelements", QList<unsigned int>());
+    m_invalidElements[path] = QSet<unsigned int>(list.begin(), list.end());
+
+    for (const auto &key : imageGroup.keyList()) {
+        bool ok = false;
+        if (ok) {
+            const QRectF rect = imageGroup.readEntry(key, QRectF());
+            m_localRectCache.insert(key.toUInt(), rect);
+        }
+    }
+}
+
+void SvgRectsCache::dropImageFromCache(const QString &path)
+{
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+    imageGroup.deleteGroup();
+    QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+}
+
+QList<QSize> SvgRectsCache::sizeHintsForId(const QString &path, const QString &id)
+{
+    const QString pathId = path % id;
+
+    if (!m_sizeHintsForId.contains(pathId)) {
+        KConfigGroup imageGroup(m_svgElementsCache, path);
+        const QStringList &encoded = imageGroup.readEntry(id, QStringList());
+        QList<QSize> sizes;
+        for (const auto &token : encoded) {
+            const auto &parts = token.split(QLatin1Char('x'));
+            if (parts.size() != 2) {
+                continue;
+            }
+            QSize size = QSize(parts[0].toDouble(), parts[1].toDouble());
+            if (!size.isEmpty()) {
+                sizes << size;
+            }
+        }
+        m_sizeHintsForId[pathId] = sizes;
+        return sizes;
+    }
+
+    return m_sizeHintsForId.value(pathId);
+}
+
+void SvgRectsCache::insertSizeHintForId(const QString &path, const QString &id, const QSize &size)
+{
+    //TODO: need to make this more efficient
+    auto sizeListToString = [] (const QList<QSize> &list) {
+        QString ret;
+        for (const auto &s : list) {
+            ret += QString::number(s.width()) % QLatin1Char('x') % QString::number(s.height()) % QLatin1Char(',');
+        }
+        return ret;
+    };
+    m_sizeHintsForId[path % id].append(size);
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+    imageGroup.writeEntry(id, sizeListToString(m_sizeHintsForId[path % id]));
+    QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+}
+
+QString SvgRectsCache::iconThemePath()
+{
+    if (!m_iconThemePath.isEmpty()) {
+        return m_iconThemePath;
+    }
+
+    KConfigGroup imageGroup(m_svgElementsCache, QStringLiteral("General"));
+    m_iconThemePath = imageGroup.readEntry(QStringLiteral("IconThemePath"), QString());
+
+    return m_iconThemePath;
+}
+
+void SvgRectsCache::setIconThemePath(const QString &path)
+{
+    m_iconThemePath = path;
+    KConfigGroup imageGroup(m_svgElementsCache, QStringLiteral("General"));
+    imageGroup.writeEntry(QStringLiteral("IconThemePath"), path);
+    QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+}
+
+void SvgRectsCache::expireCache(const QString &path)
+{
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+
+    unsigned int savedTime = imageGroup.readEntry("LastModified", QDateTime().toSecsSinceEpoch());
+    QFileInfo info(path);
+    if (info.exists()) {
+        unsigned int lastModified = info.lastModified().toSecsSinceEpoch();
+        if (lastModified <= savedTime) {
+            return;
+        }
+    }
+
+    imageGroup.deleteGroup();
+}
+
+void SvgRectsCache::setNaturalSize(const QString &path, qreal scaleFactor, const QSizeF &size)
+{
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+
+    // FIXME: needs something faster, perhaps even sprintf
+    imageGroup.writeEntry(QStringLiteral("NaturalSize_") % QString::number(scaleFactor), size);
+    QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+}
+
+QSizeF SvgRectsCache::naturalSize(const QString &path, qreal scaleFactor)
+{
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+
+    // FIXME: needs something faster, perhaps even sprintf
+    return imageGroup.readEntry(QStringLiteral("NaturalSize_") % QString::number(scaleFactor), QSizeF());
+}
+
+QStringList SvgRectsCache::cachedKeysForPath(const QString &path) const
+{
+    KConfigGroup imageGroup(m_svgElementsCache, path);
+    QStringList list = imageGroup.keyList();
+    QStringList filtered;
+
+    std::copy_if (list.begin(), list.end(), std::back_inserter(filtered), [](const QString element){bool ok; element.toLong(&ok); return ok;} );
+    return filtered;
+}
+
+void SvgRectsCache::updateLastModified(const QString &filePath, unsigned int lastModified)
+{
+    KConfigGroup imageGroup(m_svgElementsCache, filePath);
+    imageGroup.writeEntry("LastModified", lastModified);
+    QMetaObject::invokeMethod(m_configSyncTimer, QOverload<>::of(&QTimer::start));
+}
 
 SvgPrivate::SvgPrivate(Svg *svg)
     : q(svg),
@@ -166,19 +389,17 @@ SvgPrivate::~SvgPrivate()
 }
 
 //This function is meant for the rects cache
-QString SvgPrivate::cacheId(const QString &elementId) const
+SvgPrivate::CacheId SvgPrivate::cacheId(const QString &elementId) const
 {
-    if (size.isValid() && size != naturalSize) {
-        return CACHE_ID_WITH_SIZE(size, elementId, status, devicePixelRatio);
-    } else {
-        return CACHE_ID_NATURAL_SIZE(elementId, status, devicePixelRatio);
-    }
+    auto idSize = size.isValid() && size != naturalSize ? size : QSizeF{-1.0, -1.0};
+    return CacheId{idSize.width(), idSize.height(), path, elementId, status, devicePixelRatio, scaleFactor, -1, 0, lastModified};
 }
 
 //This function is meant for the pixmap cache
-QString SvgPrivate::cachePath(const QString &path, const QSize &size) const
+QString SvgPrivate::cachePath(const QString &id, const QSize &size) const
 {
-    return CACHE_ID_WITH_SIZE(size, path, status, devicePixelRatio) % QLSEP % QString::number(colorGroup);
+    auto cacheId = CacheId{double(size.width()), double(size.height()), path, id, status, devicePixelRatio, scaleFactor, colorGroup, 0, lastModified};
+    return QString::number(qHash(cacheId, SvgRectsCache::s_seed));
 }
 
 bool SvgPrivate::setImagePath(const QString &imagePath)
@@ -220,8 +441,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     themed = isThemed;
     path.clear();
     themePath.clear();
-    localRectCache.clear();
-    elementsWithSizeHints.clear();
+
     bool oldFromCurrentTheme = fromCurrentTheme;
     fromCurrentTheme = !inIconTheme && isThemed && actualTheme()->currentThemeHasImage(imagePath);
 
@@ -247,6 +467,12 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
 #endif
     }
 
+    QFileInfo info(path);
+
+    lastModified = info.lastModified().toSecsSinceEpoch();
+
+    SvgRectsCache::instance()->loadImageFromCache(path, lastModified);
+
     // check if svg wants colorscheme applied
     checkColorHints();
 
@@ -255,23 +481,14 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     if ((themed && !path.isEmpty() && QFileInfo::exists(path)) || QFileInfo::exists(actualPath)) {
         QRectF rect;
 
-        if (cacheAndColorsTheme()->findInRectsCache(path, QStringLiteral("_Natural_%1").arg(scaleFactor), rect)) {
-            naturalSize = rect.size();
-        } else {
+        naturalSize = SvgRectsCache::instance()->naturalSize(path, scaleFactor);
+        if (naturalSize.isEmpty()) {
             createRenderer();
             naturalSize = renderer->defaultSize() * scaleFactor;
-            //qCDebug(LOG_PLASMA) << "natural size for" << path << "from renderer is" << naturalSize;
-            cacheAndColorsTheme()->insertIntoRectsCache(path, QStringLiteral("_Natural_%1").arg(scaleFactor), QRectF(QPointF(0, 0), naturalSize));
-            //qCDebug(LOG_PLASMA) << "natural size for" << path << "from cache is" << naturalSize;
+            SvgRectsCache::instance()->setNaturalSize(path, scaleFactor, naturalSize);
         }
     }
 
-    if (!themed) {
-        QFile f(actualPath);
-        QFileInfo info(f);
-        lastModified = info.lastModified().toSecsSinceEpoch();
-    }
-
     q->resize();
     emit q->imagePathChanged();
 
@@ -307,39 +524,15 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
     QSize size;
     QString actualElementId;
 
-    if (elementsWithSizeHints.isEmpty()) {
-        // Fetch all size hinted element ids from the theme's rect cache
-        // and store them locally.
-        const QRegularExpression sizeHintedKeyExpr(QLatin1String("^") + CACHE_ID_NATURAL_SIZE(QStringLiteral("(\\d+)-(\\d+)-(.+)"), status, ratio) + QLatin1String("$"));
-
-	const auto lst = cacheAndColorsTheme()->listCachedRectKeys(path);
-        for (const QString &key : lst) {
-            const auto match = sizeHintedKeyExpr.match(key);
-            if (match.hasMatch()) {
-                QString baseElementId = match.captured(3);
-                QSize sizeHint(match.capturedRef(1).toInt(),
-                               match.capturedRef(2).toInt());
-                if (sizeHint.isValid()) {
-                    elementsWithSizeHints.insert(baseElementId, sizeHint);
-                }
-            }
-        }
-
-        if (elementsWithSizeHints.isEmpty()) {
-            // Make sure we won't query the theme unnecessarily.
-            elementsWithSizeHints.insert(QString(), QSize());
-        }
-    }
-
     // Look at the size hinted elements and try to find the smallest one with an
     // identical aspect ratio.
     if (s.isValid() && !elementId.isEmpty()) {
-        const QList<QSize> elementSizeHints = elementsWithSizeHints.values(elementId);
+        const QList<QSize> elementSizeHints = SvgRectsCache::instance()->sizeHintsForId(path, elementId);
 
         if (!elementSizeHints.isEmpty()) {
-            QSize bestFit(-1, -1);
+            QSizeF bestFit(-1, -1);
 
-            for (const QSize &hint : elementSizeHints) {
+            for (const auto &hint : elementSizeHints) {
 
                 if (hint.width() >= s.width() * ratio && hint.height() >= s.height() * ratio &&
                         (!bestFit.isValid() ||
@@ -369,9 +562,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
         return QPixmap();
     }
 
-    const QString id = cachePath(path, size) + actualElementId;
-
-    //qCDebug(LOG_PLASMA) << "id is " << id;
+    const QString id = cachePath(actualElementId, size);
 
     QPixmap p;
     if (cacheRendering && cacheAndColorsTheme()->findInCache(id, p, lastModified)) {
@@ -380,11 +571,6 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
         return p;
     }
 
-    //qCDebug(LOG_PLASMA) << "didn't find cached version of " << id << ", so re-rendering";
-
-    //qCDebug(LOG_PLASMA) << "size for " << actualElementId << " is " << s;
-    // we have to re-render this puppy
-
     createRenderer();
 
     QRectF finalRect = makeUniform(renderer->boundsOnElement(actualElementId), QRect(QPoint(0, 0), size));
@@ -413,9 +599,11 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
     }
 
     if (cacheRendering) {
-        cacheAndColorsTheme()->insertIntoCache(id, p, QString::number((qint64)q, 16) % QLSEP % actualElementId);
+        cacheAndColorsTheme()->insertIntoCache(id, p, QString::number((qint64)q, 16) % QLatin1Char('_') % actualElementId);
     }
 
+    SvgRectsCache::instance()->updateLastModified(path, lastModified);
+
     return p;
 }
 
@@ -425,7 +613,6 @@ void SvgPrivate::createRenderer()
         return;
     }
 
-    //qCDebug(LOG_PLASMA) << kBacktrace();
     if (themed && path.isEmpty() && !themeFailed) {
         Applet *applet = qobject_cast<Applet *>(q->parent());
         //FIXME: this maybe could be more efficient if we knew if the package was empty, e.g. for
@@ -449,17 +636,12 @@ void SvgPrivate::createRenderer()
         }
     }
 
-    //qCDebug(LOG_PLASMA) << "********************************";
-    //qCDebug(LOG_PLASMA) << "FAIL! **************************";
-    //qCDebug(LOG_PLASMA) << path << "**";
-
     QString styleSheet = cacheAndColorsTheme()->d->svgStyleSheet(colorGroup, status);
     styleCrc = qChecksum(styleSheet.toUtf8().constData(), styleSheet.size());
 
     QHash<QString, SharedSvgRenderer::Ptr>::const_iterator it = s_renderers.constFind(styleCrc + path);
 
     if (it != s_renderers.constEnd()) {
-        //qCDebug(LOG_PLASMA) << "gots us an existing one!";
         renderer = it.value();
     } else {
         if (path.isEmpty()) {
@@ -471,14 +653,19 @@ void SvgPrivate::createRenderer()
             // Add interesting elements to the theme's rect cache.
             QHashIterator<QString, QRectF> i(interestingElements);
 
+            QRegularExpression sizeHintedKeyExpr(QStringLiteral("^(\\d+)-(\\d+)-(.+)$"));
+
             while (i.hasNext()) {
                 i.next();
                 const QString &elementId = i.key();
+                QString originalId = i.key();
                 const QRectF &elementRect = i.value();
 
-                const QString cacheId = CACHE_ID_NATURAL_SIZE(elementId, status, devicePixelRatio);
-                localRectCache.insert(cacheId, elementRect);
-                cacheAndColorsTheme()->insertIntoRectsCache(path, cacheId, elementRect);
+                originalId.replace(sizeHintedKeyExpr, QStringLiteral("\\3"));
+                SvgRectsCache::instance()->insertSizeHintForId(path, originalId, elementRect.size().toSize());
+
+                const CacheId cacheId({-1.0, -1.0, path, elementId, status, devicePixelRatio, scaleFactor, -1, 0, lastModified});
+                SvgRectsCache::instance()->insert(cacheId, elementRect, lastModified);
             }
         }
 
@@ -493,23 +680,13 @@ void SvgPrivate::createRenderer()
 void SvgPrivate::eraseRenderer()
 {
     if (renderer &&
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
         renderer->ref.loadRelaxed() == 2) {
-#else
-        renderer->ref.load() == 2) {
-#endif
         // this and the cache reference it
         s_renderers.erase(s_renderers.find(styleCrc + path));
-
-        if (theme) {
-            theme.data()->releaseRectsCache(path);
-        }
     }
 
     renderer = nullptr;
     styleCrc = 0;
-    localRectCache.clear();
-    elementsWithSizeHints.clear();
 }
 
 QRectF SvgPrivate::elementRect(const QString &elementId)
@@ -531,35 +708,23 @@ QRectF SvgPrivate::elementRect(const QString &elementId)
         return QRectF();
     }
 
-    const QString id = cacheId(elementId);
-    const auto it = localRectCache.constFind(id);
-    if (it != localRectCache.constEnd()) {
-        return *it;
-    }
-
     QRectF rect;
-    bool found = cacheAndColorsTheme()->findInRectsCache(path, id, rect);
+    const CacheId cacheId = SvgPrivate::cacheId(elementId);
+    bool found = SvgRectsCache::instance()->findElementRect(cacheId, rect);
     //This is a corner case where we are *sure* the element is not valid
-    if (found && rect == QRectF()) {
-        return rect;
-    } else if (found) {
-        localRectCache.insert(id, rect);
-    } else {
-        rect = findAndCacheElementRect(elementId, id);
+    if (!found) {
+        rect = findAndCacheElementRect(elementId);
     }
 
     return rect;
 }
 
-QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId, const QString &id)
+QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId)
 {
     //we need to check the id before createRenderer(), otherwise it may generate a different id compared to the previous cacheId)( call
-    createRenderer();
+    const CacheId cacheId = SvgPrivate::cacheId(elementId);
 
-    const auto it = localRectCache.constFind(id);
-    if (it != localRectCache.constEnd()) {
-        return *it;
-    }
+    createRenderer();
 
     //This code will usually never be run because createRenderer already caches all the boundingRect in the elements in the svg
     QRectF elementRect = renderer->elementExists(elementId) ?
@@ -576,9 +741,7 @@ QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId, const QStri
 
     elementRect = QRectF(elementRect.x() * dx, elementRect.y() * dy,
                          elementRect.width() * dx, elementRect.height() * dy);
-
-    cacheAndColorsTheme()->insertIntoRectsCache(path, id, elementRect);
-    localRectCache.insert(id, elementRect);
+    SvgRectsCache::instance()->insert(cacheId, elementRect, lastModified);
 
     return elementRect;
 }
@@ -658,7 +821,6 @@ QRectF SvgPrivate::makeUniform(const QRectF &orig, const QRectF &dst)
         res.setHeight(res.height() + offset);
     }
 
-    //qCDebug(LOG_PLASMA)<<"Aligning Rects, origin:"<<orig<<"destination:"<<dst<<"result:"<<res;
     return res;
 }
 
@@ -748,9 +910,8 @@ void Svg::setScaleFactor(qreal ratio)
     //not resize() because we want to do it unconditionally
     QRectF rect;
 
-    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QStringLiteral("_Natural_%1").arg(d->scaleFactor), rect)) {
-        d->naturalSize = rect.size();
-    } else {
+    d->naturalSize = SvgRectsCache::instance()->naturalSize(d->path, d->scaleFactor);
+    if (d->naturalSize.isEmpty()) {
         d->createRenderer();
         d->naturalSize = d->renderer->defaultSize() * d->scaleFactor;
     }
@@ -856,7 +1017,6 @@ void Svg::resize(const QSizeF &size)
     }
 
     d->size = size;
-    d->localRectCache.clear();
     emit sizeChanged();
 }
 
@@ -868,7 +1028,6 @@ void Svg::resize()
     }
 
     d->size = d->naturalSize;
-    d->localRectCache.clear();
     emit sizeChanged();
 }
 
@@ -884,7 +1043,7 @@ QRectF Svg::elementRect(const QString &elementId) const
 
 bool Svg::hasElement(const QString &elementId) const
 {
-    if (d->path.isNull() && d->themePath.isNull()) {
+    if (elementId.isEmpty() || (d->path.isNull() && d->themePath.isNull())) {
         return false;
     }
 
@@ -899,14 +1058,14 @@ bool Svg::isValid() const
 
     //try very hard to avoid creation of a parser
     QRectF rect;
-    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QStringLiteral("_Natural_%1").arg(d->scaleFactor), rect)) {
+    QSizeF naturalSize = SvgRectsCache::instance()->naturalSize(d->path, d->scaleFactor);
+    if (!naturalSize.isEmpty()) {
         return true;
     }
 
     if (d->path.isEmpty() || !QFileInfo::exists(d->path)) {
         return false;
     }
-
     d->createRenderer();
     return d->renderer->isValid();
 }
@@ -924,7 +1083,6 @@ bool Svg::containsMultipleImages() const
 void Svg::setImagePath(const QString &svgFilePath)
 {
     if (d->setImagePath(svgFilePath)) {
-        //qCDebug(LOG_PLASMA) << "repaintNeeded";
         emit repaintNeeded();
     }
 }
