@@ -6,9 +6,9 @@
 
 #include "theme.h"
 #include "private/theme_p.h"
+#include "private/svg_p.h"
 
 #include <QFile>
-#include <QFont>
 #include <QFontDatabase>
 #include <QFileInfo>
 #include <QMutableListIterator>
@@ -25,7 +25,6 @@
 #include <QDebug>
 #include <KDirWatch>
 #include <KImageCache>
-#include <KSharedConfig>
 #include <KWindowEffects>
 #include <KWindowSystem>
 #include <QStandardPaths>
@@ -40,11 +39,11 @@ Theme::Theme(QObject *parent)
 {
     if (!ThemePrivate::globalTheme) {
         ThemePrivate::globalTheme = new ThemePrivate;
+        ThemePrivate::globalTheme->settingsChanged(false);
     }
     ThemePrivate::globalTheme->ref.ref();
     d = ThemePrivate::globalTheme;
 
-    d->settingsChanged(false);
     if (QCoreApplication::instance()) {
         connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
                 d, &ThemePrivate::onAppExitCleanup);
@@ -282,23 +281,31 @@ bool Theme::useGlobalSettings() const
 
 bool Theme::findInCache(const QString &key, QPixmap &pix, unsigned int lastModified)
 {
-    if (lastModified != 0 && d->useCache() && lastModified > uint(d->pixmapCache->lastModifiedTime().toSecsSinceEpoch())) {
+    //TODO KF6: Make lastModified non-optional.
+    if (lastModified == 0) {
+        qCWarning(LOG_PLASMA) << "findInCache with a lastModified timestamp of 0 is deprecated";
         return false;
     }
 
-    if (d->useCache()) {
-        const QString id = d->keysToCache.value(key);
-        const auto it = d->pixmapsToCache.constFind(id);
-        if (it != d->pixmapsToCache.constEnd()) {
-            pix = *it;
-            return !pix.isNull();
-        }
+    if (!d->useCache()) {
+        return false;
+    }
 
-        QPixmap temp;
-        if (d->pixmapCache->findPixmap(key, &temp) && !temp.isNull()) {
-            pix = temp;
-            return true;
-        }
+    if (lastModified > uint(d->pixmapCache->lastModifiedTime().toSecsSinceEpoch())) {
+        return false;
+    }
+
+    const QString id = d->keysToCache.value(key);
+    const auto it = d->pixmapsToCache.constFind(id);
+    if (it != d->pixmapsToCache.constEnd()) {
+        pix = *it;
+        return !pix.isNull();
+    }
+
+    QPixmap temp;
+    if (d->pixmapCache->findPixmap(key, &temp) && !temp.isNull()) {
+        pix = temp;
+        return true;
     }
 
     return false;
@@ -329,36 +336,13 @@ bool Theme::findInRectsCache(const QString &image, const QString &element, QRect
         return false;
     }
 
-    KConfigGroup imageGroup(d->svgElementsCache, image);
-    rect = imageGroup.readEntry(element % QLatin1String("Size"), QRectF());
-
-    if (rect.isValid()) {
-        return true;
-    }
-
-    //Name starting by _ means the element is empty and we're asked for the size of
-    //the whole image, so the whole image is never invalid
-    if (element.indexOf(QLatin1Char('_')) <= 0) {
+    bool ok = false;
+    uint id = element.toLong(&ok);
+    if (!ok) {
         return false;
     }
 
-    bool invalid = false;
-
-    QHash<QString, QSet<QString> >::iterator it = d->invalidElements.find(image);
-    if (it == d->invalidElements.end()) {
-        const QStringList elementList = imageGroup.readEntry("invalidElements", QStringList());
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
-        const QSet<QString> elements(elementList.begin(), elementList.end());
-#else
-        const QSet<QString> elements = elementList.toSet();
-#endif
-        d->invalidElements.insert(image, elements);
-        invalid = elements.contains(element);
-    } else {
-        invalid = it.value().contains(element);
-    }
-
-    return invalid;
+    return SvgRectsCache::instance()->findElementRect(id, image, rect);
 }
 
 QStringList Theme::listCachedRectKeys(const QString &image) const
@@ -367,21 +351,7 @@ QStringList Theme::listCachedRectKeys(const QString &image) const
         return QStringList();
     }
 
-    KConfigGroup imageGroup(d->svgElementsCache, image);
-    QStringList keys = imageGroup.keyList();
-
-    QMutableListIterator<QString> i(keys);
-    while (i.hasNext()) {
-        QString key = i.next();
-        if (key.endsWith(QLatin1String("Size"))) {
-            // The actual cache id used from outside doesn't end on "Size".
-            key.resize(key.size() - 4);
-            i.setValue(key);
-        } else {
-            i.remove();
-        }
-    }
-    return keys;
+    return SvgRectsCache::instance()->cachedKeysForPath(image);
 }
 
 void Theme::insertIntoRectsCache(const QString &image, const QString &element, const QRectF &rect)
@@ -390,46 +360,26 @@ void Theme::insertIntoRectsCache(const QString &image, const QString &element, c
         return;
     }
 
-    if (rect.isValid()) {
-        KConfigGroup imageGroup(d->svgElementsCache, image);
-        imageGroup.writeEntry(element % QLatin1String("Size"), rect);
-    } else {
-        QHash<QString, QSet<QString> >::iterator it = d->invalidElements.find(image);
-        if (it == d->invalidElements.end()) {
-            d->invalidElements[image].insert(element);
-        } else if (!it.value().contains(element)) {
-            if (it.value().count() > 1000) {
-                it.value().erase(it.value().begin());
-            }
-
-            it.value().insert(element);
-        }
+    bool ok = false;
+    uint id = element.toLong(&ok);
+    if (!ok) {
+        return;
     }
 
-    QMetaObject::invokeMethod(d->rectSaveTimer, "start");
+    uint secs = QDateTime::currentSecsSinceEpoch();
+    SvgRectsCache::instance()->insert(id, image, rect, secs);
 }
 
 void Theme::invalidateRectsCache(const QString &image)
 {
-    if (d->useCache()) {
-        KConfigGroup imageGroup(d->svgElementsCache, image);
-        imageGroup.deleteGroup();
-    }
-
-    d->invalidElements.remove(image);
+    
+    SvgRectsCache::instance()->dropImageFromCache(image);
 }
 
 void Theme::releaseRectsCache(const QString &image)
 {
-    QHash<QString, QSet<QString> >::iterator it = d->invalidElements.find(image);
-    if (it != d->invalidElements.end()) {
-        if (d->useCache()) {
-            KConfigGroup imageGroup(d->svgElementsCache, it.key());
-            imageGroup.writeEntry("invalidElements", it.value().values());
-        }
-
-        d->invalidElements.erase(it);
-    }
+    Q_UNUSED(image);
+    // No op: the internal svg cache always writes the invalid elements in the proper place
 }
 
 void Theme::setCacheLimit(int kbytes)
