@@ -1,36 +1,23 @@
 /*
- *   Copyright 2014 Marco Martin <mart@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2014 Marco Martin <mart@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "appletquickitem.h"
 #include "private/appletquickitem_p.h"
 #include "debug_p.h"
 
-#include <QQmlComponent>
+#include <QJsonArray>
 #include <QQmlExpression>
-#include <QQmlEngine>
 #include <QQmlProperty>
 #include <QQmlContext>
 #include <QQuickWindow>
+#include <QRandomGenerator>
 
 #include <QDebug>
 
-#include <klocalizedstring.h>
+#include <KLocalizedString>
 
 #include <Plasma/Applet>
 #include <Plasma/Containment>
@@ -53,7 +40,8 @@ AppletQuickItemPrivate::AppletQuickItemPrivate(Plasma::Applet *a, AppletQuickIte
       switchHeight(-1),
       applet(a),
       expanded(false),
-      activationTogglesExpanded(false)
+      activationTogglesExpanded(true),
+      initComplete(false)
 {
     if (s_preloadPolicy == Uninitialized) {
         //default as Adaptive
@@ -61,9 +49,9 @@ AppletQuickItemPrivate::AppletQuickItemPrivate(Plasma::Applet *a, AppletQuickIte
 
         if (qEnvironmentVariableIsSet("PLASMA_PRELOAD_POLICY")) {
             const QString policy = QString::fromUtf8(qgetenv("PLASMA_PRELOAD_POLICY")).toLower();
-            if (policy == QStringLiteral("aggressive")) {
+            if (policy == QLatin1String("aggressive")) {
                 s_preloadPolicy = Aggressive;
-            } else if (policy == QStringLiteral("none")) {
+            } else if (policy == QLatin1String("none")) {
                 s_preloadPolicy = None;
             }
         }
@@ -94,7 +82,7 @@ int AppletQuickItemPrivate::preloadWeight() const
     const QStringList provides(KPluginMetaData::readStringList(applet->pluginMetaData().rawData(), QStringLiteral("X-Plasma-Provides")));
 
     //some applet types we want a bigger weight
-    if (provides.contains(QStringLiteral("org.kde.plasma.launchermenu"))) {
+    if (provides.contains(QLatin1String("org.kde.plasma.launchermenu"))) {
         defaultWeight = DefaultLauncherPreloadWeight;
     } else {
         defaultWeight = DefaultPreloadWeight;
@@ -115,7 +103,8 @@ void AppletQuickItemPrivate::connectLayoutAttached(QObject *item)
 
     //Search a child that has the needed Layout properties
     //HACK: here we are not type safe, but is the only way to access to a pointer of Layout
-    foreach (QObject *child, item->children()) {
+    const auto lstChildren = item->children();
+    for (QObject *child : lstChildren) {
         //find for the needed property of Layout: minimum/maximum/preferred sizes and fillWidth/fillHeight
         if (child->property("minimumWidth").isValid() && child->property("minimumHeight").isValid() &&
                 child->property("preferredWidth").isValid() && child->property("preferredHeight").isValid() &&
@@ -156,7 +145,8 @@ void AppletQuickItemPrivate::connectLayoutAttached(QObject *item)
 
     QObject *ownLayout = nullptr;
 
-    foreach (QObject *child, q->children()) {
+    const auto children = q->children();
+    for (QObject *child : children) {
         //find for the needed property of Layout: minimum/maximum/preferred sizes and fillWidth/fillHeight
         if (child->property("minimumWidth").isValid() && child->property("minimumHeight").isValid() &&
                 child->property("preferredWidth").isValid() && child->property("preferredHeight").isValid() &&
@@ -328,7 +318,7 @@ void AppletQuickItemPrivate::preloadForExpansion()
 
     if (!appletShouldBeExpanded() && compactRepresentationExpanderItem) {
         compactRepresentationExpanderItem->setProperty("fullRepresentation", QVariant::fromValue<QObject*>(createFullRepresentationItem()));
-    } else {
+    } else if (fullRepresentationItem) {
         fullRepresentationItem->setProperty("parent", QVariant::fromValue<QObject*>(q));
     }
 
@@ -337,11 +327,15 @@ void AppletQuickItemPrivate::preloadForExpansion()
         fullRepresentationItem->window()->create();
     }
 
-    qCInfo(LOG_PLASMAQUICK) << "Applet" << applet->title() << "loaded after" << ( QDateTime::currentMSecsSinceEpoch() - time) << "msec";
+    qCDebug(LOG_PLASMAQUICK) << "Applet" << applet->title() << "loaded after" << ( QDateTime::currentMSecsSinceEpoch() - time) << "msec";
 }
 
 void AppletQuickItemPrivate::compactRepresentationCheck()
 {
+    if (!initComplete) {
+        return;
+    }
+
     if (!qmlObject->rootObject()) {
         return;
     }
@@ -515,6 +509,8 @@ AppletQuickItem::~AppletQuickItem()
     delete d->compactRepresentationExpanderItem;
 
     AppletQuickItemPrivate::s_rootObjects.remove(d->qmlObject->rootContext());
+
+    delete d;
 }
 
 AppletQuickItem *AppletQuickItem::qmlAttachedProperties(QObject *object)
@@ -595,15 +591,26 @@ void AppletQuickItem::init()
 
     if (!engine || !engine->rootContext() || !engine->rootContext()->isValid() || !d->qmlObject->mainComponent() || d->qmlObject->mainComponent()->isError() || d->applet->failedToLaunch()) {
         QString reason;
+        QJsonObject errorData;
+        errorData[QStringLiteral("appletName")] = i18n("Unknown Applet");
+        errorData[QStringLiteral("isDebugMode")] = qEnvironmentVariableIntValue("PLASMA_ENABLE_QML_DEBUG") != 0;
+
         if (d->applet->failedToLaunch()) {
             reason = d->applet->launchErrorMessage();
+            errorData[QStringLiteral("errors")] = QJsonArray::fromStringList({reason});
         } else if (d->applet->kPackage().isValid()) {
-            foreach (QQmlError error, d->qmlObject->mainComponent()->errors()) {
+            const auto errors = d->qmlObject->mainComponent()->errors();
+            QStringList errorList;
+            for (const QQmlError &error : errors) {
                 reason += error.toString() + QLatin1Char('\n');
+                errorList << error.toString();
             }
+            errorData[QStringLiteral("errors")] = QJsonArray::fromStringList(errorList);
+            errorData[QStringLiteral("appletName")] = d->applet->kPackage().metadata().name();
             reason = i18n("Error loading QML file: %1", reason);
         } else {
             reason = i18n("Error loading Applet: package inexistent. %1", applet()->launchErrorMessage());
+            errorData[QStringLiteral("errors")] = QJsonArray::fromStringList({reason});
         }
 
         d->qmlObject->setSource(d->coronaPackage.fileUrl("appleterror"));
@@ -613,6 +620,8 @@ void AppletQuickItem::init()
         if (d->qmlObject->mainComponent()->isError()) {
             return;
         } else {
+            d->qmlObject->rootObject()->setProperty("errorInformation", errorData);
+            // TODO KF6: remove in favour of newer errorInformation
             d->qmlObject->rootObject()->setProperty("reason", reason);
         }
 
@@ -667,6 +676,7 @@ void AppletQuickItem::init()
         d->compactRepresentationExpander->loadUrl(compactExpanderUrl);
     }
 
+    d->initComplete = true;
     d->compactRepresentationCheck();
     qmlObject()->engine()->rootContext()->setBaseUrl(qmlObject()->source());
     qmlObject()->engine()->setContextForObject(this, qmlObject()->engine()->rootContext());
@@ -683,9 +693,9 @@ void AppletQuickItem::init()
             this, [this](bool uiReady) {
                 if (uiReady && d->s_preloadPolicy >= AppletQuickItemPrivate::Adaptive) {
                     const int preloadWeight = d->preloadWeight();
-                    qCInfo(LOG_PLASMAQUICK) << "New Applet " << d->applet->title() << "with a weight of" << preloadWeight;
+                    qCDebug(LOG_PLASMAQUICK) << "New Applet " << d->applet->title() << "with a weight of" << preloadWeight;
 
-                    //don't preload applets less then a certain weigth
+                    //don't preload applets less then a certain weight
                     if (d->s_preloadPolicy >= AppletQuickItemPrivate::Aggressive || preloadWeight >= AppletQuickItemPrivate::DelayedPreloadWeight) {
                         //spread the creation over a random delay to make it look
                         //plasma started already, and load the popup in the background
@@ -693,9 +703,9 @@ void AppletQuickItem::init()
                         //to be the delay, smaller minimum walue, smaller spread
                         const int min = (100 - preloadWeight) * 20;
                         const int max = (100 - preloadWeight) * 100;
-                        const int delay = qrand() % ((max + 1) - min) + min;
+                        const int delay = QRandomGenerator::global()->bounded((max + 1) - min) + min;
                         QTimer::singleShot(delay, this, [this, delay]() {
-                            qCInfo(LOG_PLASMAQUICK) << "Delayed preload of " << d->applet->title() << "after" << (qreal)delay/1000 << "seconds";
+                            qCDebug(LOG_PLASMAQUICK) << "Delayed preload of " << d->applet->title() << "after" << (qreal)delay/1000 << "seconds";
                             d->preloadForExpansion();
                         });
                     }
@@ -737,6 +747,7 @@ void AppletQuickItem::setSwitchWidth(int width)
     }
 
     d->switchWidth = width;
+    d->compactRepresentationCheck();
     emit switchWidthChanged(width);
 }
 
@@ -752,6 +763,7 @@ void AppletQuickItem::setSwitchHeight(int height)
     }
 
     d->switchHeight = height;
+    d->compactRepresentationCheck();
     emit switchHeightChanged(height);
 }
 
@@ -835,7 +847,7 @@ void AppletQuickItem::setExpanded(bool expanded)
         if (d->s_preloadPolicy >= AppletQuickItemPrivate::Adaptive && !d->applet->isContainment()) {
             const int newWeight = qMin(d->preloadWeight() + AppletQuickItemPrivate::PreloadWeightIncrement, 100);
             d->applet->config().writeEntry(QStringLiteral("PreloadWeight"), newWeight);
-            qCInfo(LOG_PLASMAQUICK) << "Increasing score for" << d->applet->title() << "to" << newWeight;
+            qCDebug(LOG_PLASMAQUICK) << "Increasing score for" << d->applet->title() << "to" << newWeight;
         }
     }
 
