@@ -1,24 +1,11 @@
-/***************************************************************************
- *   Copyright 2011 Marco Martin <mart@kde.org>                            *
- *   Copyright 2013 Sebastian Kügler <sebas@kde.org>                       *
- *   Copyright 2014 Martin Gräßlin <mgraesslin@kde.org>                    *
- *   Copyright 2014 Vishesh Handa <vhanda@kde.org>                         *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2011 Marco Martin <mart@kde.org>
+    SPDX-FileCopyrightText: 2013 Sebastian Kügler <sebas@kde.org>
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2014 Vishesh Handa <vhanda@kde.org>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "dialog.h"
 #include "config-plasma.h"
@@ -36,13 +23,12 @@
 
 #include <QPlatformSurfaceEvent>
 
-#include <kwindowsystem.h>
+#include <KWindowSystem>
 #include <KWindowSystem/KWindowInfo>
 
 #include <kquickaddons/quickviewsharedengine.h>
 
-#include <kwindoweffects.h>
-#include <Plasma/Plasma>
+#include <KWindowEffects>
 #include <Plasma/Corona>
 
 #include <QDebug>
@@ -179,7 +165,8 @@ QRect DialogPrivate::availableScreenGeometryForPosition(const QPoint& pos) const
     //        we simply iterate over the virtual screens and pick the one our QWindow
     //        says it's at.
     QRect avail;
-    Q_FOREACH (QScreen *screen, QGuiApplication::screens()) {
+    const auto screens = QGuiApplication::screens();
+    for (QScreen *screen : screens) {
         //we check geometry() but then take availableGeometry()
         //to reliably check in which screen a position is, we need the full
         //geometry, including areas for panels
@@ -875,7 +862,8 @@ void Dialog::setMainItem(QQuickItem *mainItem)
 
             //Search a child that has the needed Layout properties
             //HACK: here we are not type safe, but is the only way to access to a pointer of Layout
-            foreach (QObject *child, mainItem->children()) {
+            const auto lstChild = mainItem->children();
+            for (QObject *child : lstChild) {
                 //find for the needed property of Layout: minimum/maximum/preferred sizes and fillWidth/fillHeight
                 if (child->property("minimumWidth").isValid() && child->property("minimumHeight").isValid() &&
                         child->property("preferredWidth").isValid() && child->property("preferredHeight").isValid() &&
@@ -1091,8 +1079,15 @@ QObject *Dialog::margins() const
     return d->frameSvgItem->fixedMargins();
 }
 
+QObject *Dialog::inset() const
+{
+    return d->frameSvgItem->inset();
+}
+
 void Dialog::setFramelessFlags(Qt::WindowFlags flags)
 {
+    if (d->type == Dialog::Normal)
+        flags |= Qt::Dialog;
     setFlags(Qt::FramelessWindowHint | flags);
     d->applyType();
     emit flagsChanged();
@@ -1177,7 +1172,7 @@ void Dialog::focusOutEvent(QFocusEvent *ev)
         }
 
         const QWindow *focusWindow = QGuiApplication::focusWindow();
-        bool childHasFocus = focusWindow && ((focusWindow->isActive() && isAncestorOf(focusWindow)) || focusWindow->type() & Qt::Popup);
+        bool childHasFocus = focusWindow && ((focusWindow->isActive() && isAncestorOf(focusWindow)) || (focusWindow->type() & Qt::Popup) == Qt::Popup);
 
         const bool viewClicked = qobject_cast<const KQuickAddons::QuickViewSharedEngine *>(focusWindow) || qobject_cast<const View *>(focusWindow) || qobject_cast<const ConfigView *>(focusWindow);
 
@@ -1196,7 +1191,7 @@ void Dialog::showEvent(QShowEvent *event)
         DialogShadows::self()->addWindow(this, d->frameSvgItem->enabledBorders());
     }
 
-    KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
+    KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager | NET::SkipSwitcher);
 
     QQuickWindow::showEvent(event);
 }
@@ -1223,7 +1218,7 @@ bool Dialog::event(QEvent *event)
         //sometimes non null regions arrive even for non visible windows
         //for which surface creation would fail
         if (!d->shellSurface && isVisible()) {
-            KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
+            KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager | NET::SkipSwitcher);
             d->setupWaylandIntegration();
             d->updateVisibility(true);
             d->updateTheme();
@@ -1233,7 +1228,7 @@ bool Dialog::event(QEvent *event)
         const QPlatformSurfaceEvent *pSEvent = static_cast<QPlatformSurfaceEvent *>(event);
 
         if (pSEvent->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated) {
-            KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
+            KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager | NET::SkipSwitcher);
         }
     } else if (event->type() == QEvent::Show) {
         d->updateVisibility(true);
@@ -1283,24 +1278,13 @@ bool Dialog::event(QEvent *event)
         case QEvent::Wheel: {
             QWheelEvent *we = static_cast<QWheelEvent *>(event);
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-            const QPoint pos = we->pos();
-#else
             const QPoint pos = we->position().toPoint();
-#endif
 
             if (!d->mainItemContainsPosition(pos)) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-                QWheelEvent we2(d->positionAdjustedForMainItem(pos),
-                                d->positionAdjustedForMainItem(pos) + position(),
-                                we->pixelDelta(), we->angleDelta(), we->angleDelta().y(),
-                                we->orientation(), we->buttons(), we->modifiers(), we->phase());
-#else
                 QWheelEvent we2(d->positionAdjustedForMainItem(pos),
                                 d->positionAdjustedForMainItem(pos) + position(),
                                 we->pixelDelta(), we->angleDelta(),
                                 we->buttons(), we->modifiers(), we->phase(), false /*inverted*/);
-#endif
 
                 if (isVisible()) {
                     QCoreApplication::sendEvent(this, &we2);
