@@ -1,22 +1,9 @@
-/***************************************************************************
- *   Copyright 2011 Marco Martin <mart@kde.org>                            *
- *   Copyright 2013 Sebastian Kügler <sebas@kde.org>                       *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2011 Marco Martin <mart@kde.org>
+    SPDX-FileCopyrightText: 2013 Sebastian Kügler <sebas@kde.org>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 #ifndef DIALOG_PROXY_P
 #define DIALOG_PROXY_P
 
@@ -24,6 +11,7 @@
 #include <QQuickWindow>
 #include <QPoint>
 #include <QQmlParserStatus>
+#include <QScopedPointer>
 
 #include <Plasma/Plasma>
 #include <Plasma/Theme>
@@ -101,6 +89,13 @@ class PLASMAQUICK_EXPORT Dialog : public QQuickWindow, public QQmlParserStatus
      */
     Q_PROPERTY(QObject *margins READ margins CONSTANT)
 
+    /**
+     * Margins where the dialog background actually starts, excluiding things like shadows or borders
+     * @see DialogMargins
+     * @since 5.77
+     */
+    Q_PROPERTY(QObject *inset READ inset CONSTANT)
+
     /**
      * Plasma Location of the dialog window. Useful if this dialog is a popup for a panel
      */
@@ -157,7 +152,8 @@ public:
         PopupMenu = NET::PopupMenu,
         Tooltip = NET::Tooltip,
         Notification = NET::Notification,
-        OnScreenDisplay = NET::OnScreenDisplay
+        OnScreenDisplay = NET::OnScreenDisplay,
+        CriticalNotification = NET::CriticalNotification
     };
     Q_ENUM(WindowType)
 
@@ -181,6 +177,7 @@ public:
     void setLocation(Plasma::Types::Location location);
 
     QObject *margins() const;
+    QObject *inset() const;
 
     void setFramelessFlags(Qt::WindowFlags flags);
 
@@ -240,7 +237,7 @@ protected:
 
 private:
     friend class DialogPrivate;
-    DialogPrivate *const d;
+    const QScopedPointer<DialogPrivate> d;
 
     Q_PRIVATE_SLOT(d, void updateInputShape())
     Q_PRIVATE_SLOT(d, void updateTheme())
