@@ -7,12 +7,10 @@
 #ifndef APPLETQUICKITEM_P_H
 #define APPLETQUICKITEM_P_H
 
-#include <KPackage/Package>
 #include <QQmlComponent>
-#include <QQmlEngine>
 #include <QQuickItem>
-
-#include <Plasma/Corona>
+#include <QQmlEngine>
+#include <KPackage/Package>
 
 //
 //  W A R N I N G
@@ -25,37 +23,44 @@
 // We mean it.
 //
 
+
 namespace Plasma
 {
 class Applet;
 }
 
+namespace KDeclarative
+{
+class QmlObject;
+}
+
 namespace PlasmaQuick
 {
+
 class AppletQuickItem;
-class SharedQmlEngine;
-class AppletContext;
 
 class AppletQuickItemPrivate
 {
 public:
-    // weight values for the logic for when or if to preload
+    //weight values for the logic for when or if to preload
     enum PreloadWeights {
         DefaultPreloadWeight = 50,
         DefaultLauncherPreloadWeight = 100,
         DelayedPreloadWeight = 25,
         PreloadWeightIncrement = 5,
-        PreloadWeightDecrement = 8,
+        PreloadWeightDecrement = 8
     };
 
     enum PreloadPolicy {
         Uninitialized = -1,
         None = 0,
         Adaptive = 1,
-        Aggressive = 2,
+        Aggressive = 2
     };
 
-    AppletQuickItemPrivate(AppletQuickItem *item);
+    AppletQuickItemPrivate(Plasma::Applet *a, AppletQuickItem *item);
+
+    void init();
 
     int preloadWeight() const;
 
@@ -63,19 +68,17 @@ public:
     QQuickItem *createFullRepresentationItem();
     QQuickItem *createCompactRepresentationExpanderItem();
 
-    // true if the applet is at a size in which it should be expanded,
-    // false if is too small and should be an icon
+    //true if the applet is at a size in which it should be expanded,
+    //false if is too small and should be an icon
     bool appletShouldBeExpanded() const;
-    // ensures the popup is preloaded, don't expand yet
+    //ensures the popup is preloaded, don't expand yet
     void preloadForExpansion();
 
-    // look into item, and return the Layout attached property, if found
-    QObject *searchLayoutAttached(QObject *parent);
+    //look into item, and return the Layout attached property, if found
     void connectLayoutAttached(QObject *item);
     void propagateSizeHint(const QByteArray &layoutProperty);
 
-    // handlers of Layout signals, private slots
-    static void anchorsFillParent(QQuickItem *item, QQuickItem *parent);
+    //handlers of Layout signals, private slots
     void compactRepresentationCheck();
     void minimumWidthChanged();
     void minimumHeightChanged();
@@ -88,7 +91,6 @@ public:
 
     AppletQuickItem *q;
 
-    static QHash<Plasma::Applet *, AppletQuickItem *> s_itemsForApplet;
     static PreloadPolicy s_preloadPolicy;
     int switchWidth;
     int switchHeight;
@@ -104,23 +106,22 @@ public:
     QPointer<QQuickItem> currentRepresentationItem;
     QPointer<QObject> testItem;
 
-    // Attached layout objects: own and the representation's one
+    //Attached layout objects: own and the representation's one
     QPointer<QObject> representationLayout;
     QPointer<QObject> ownLayout;
 
     Plasma::Applet *applet;
-    PlasmaQuick::SharedQmlEngine *qmlObject;
+    KDeclarative::QmlObject *qmlObject;
 
     KPackage::Package appletPackage;
     KPackage::Package coronaPackage;
     KPackage::Package containmentPackage;
 
-    bool expanded = false;
-    bool hideOnWindowDeactivate = false;
-    bool preloadFullRepresentation = false;
+    bool expanded : 1;
     bool activationTogglesExpanded : 1;
-    bool initComplete : 1;
-    bool compactRepresentationCheckGuard : 1;
+    bool initComplete: 1;
+
+    static QHash<QObject *, AppletQuickItem *> s_rootObjects;
 };
 
 }
