@@ -1,23 +1,10 @@
 /*
- *   Copyright 2008 Chani Armitage <chani@kde.org>
- *   Copyright 2008, 2009 Aaron Seigo <aseigo@kde.org>
- *   Copyright 2010 Marco Martin <mart@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2008 Chani Armitage <chani@kde.org>
+    SPDX-FileCopyrightText: 2008, 2009 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2010 Marco Martin <mart@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "containmentinterface.h"
 #include "wallpaperinterface.h"
@@ -55,7 +42,6 @@
 
 #include <kactivities/info.h>
 
-#include "kdeclarative/configpropertymap.h"
 #include <packageurlinterceptor.h>
 
 ContainmentInterface::ContainmentInterface(DeclarativeAppletScript *parent, const QVariantList &args)
@@ -442,7 +428,7 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
         }
         return;
     }
-    m_dropMenu = QPointer<DropMenu>(new DropMenu(dropJob, QPoint(x, y), this));
+    m_dropMenu = QPointer<DropMenu>(new DropMenu(dropJob, mapToGlobal(QPoint(x, y)).toPoint(), this));
     if (dropJob) {
         dropJob->setParent(m_dropMenu);
     }
@@ -459,11 +445,7 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
 
     if (mimeData->hasFormat(QStringLiteral("text/x-plasmoidservicename"))) {
         QString data = QString::fromUtf8( mimeData->data(QStringLiteral("text/x-plasmoidservicename")) );
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
         const QStringList appletNames = data.split(QLatin1Char('\n'), Qt::SkipEmptyParts);
-#else
-        const QStringList appletNames = data.split(QLatin1Char('\n'), QString::SkipEmptyParts);
-#endif
         for (const QString &appletName : appletNames) {
             qDebug() << "adding" << appletName;
 
@@ -495,8 +477,8 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
         job->setParent(m_dropMenu.data());
 
         QObject::connect(job, &KJob::result, this, &ContainmentInterface::dropJobResult);
-        QObject::connect(job, SIGNAL(mimetype(KIO::Job*, QString)),
-                this, SLOT(mimeTypeRetrieved(KIO::Job*,QString)));
+        QObject::connect(job, &KIO::MimetypeJob::mimeTypeFound,
+                this, &ContainmentInterface::mimeTypeRetrieved);
 
     } else {
         bool deleteDropMenu = true;
@@ -886,6 +868,12 @@ QList<QObject *> ContainmentInterface::actions() const
     return actionList;
 }
 
+void ContainmentInterface::setContainmentDisplayHints(Plasma::Types::ContainmentDisplayHints hints)
+{
+    m_containment->setContainmentDisplayHints(hints);
+}
+
+
 //PROTECTED--------------------
 
 void ContainmentInterface::mouseReleaseEvent(QMouseEvent *event)
